#include <iostream>
#include <iomanip>
#include <math.h>
#include <string.h>
#include <stdio.h>
#include <random>
#include <fstream>
#include <sstream>
#include <algorithm>
#include "chaincmdline.h"
#include <queue>
#include <unordered_map>
#include <cstdint>
#include <limits>
#include <climits>
#include <omp.h>
#include <vector>

#define NDEBUG
#include <cassert>




using namespace std;

enum Type_S {
 rootep=0,
 twoep=1,
 starsplit=2,
 outlier=4
};

using b_double=long double; //big double
using s_double=long double; //small double


using g_int=int16_t; //geounit index
using D_int=int8_t;  //District index


#define PI 3.14159265359

#define MAXDISTRICTS 127 //maximum number of districts
#define MAXDEGREE 127
//GLOBAL VARIABLES
int g_MAXEDGES;      //theoretical maximum districting degree
int g_NUMDISTRICTS;  //number of districts in the state
//int g_MAXMUNIS;      //max label of municipalities
char g_debuglevel;   //debug output level
Type_S StatTest;     //which statistical test do we use?
int g_NUMCLUSTERS;      //number of clusters (islands, county clusters, etc)
int g_NUMPRECINCTS;  //NOT geounits.  This is precincts whose breaks we are constraining.
int g_NUMGEOS;
int g_MAXDEGREE;     //max number of geounits incident with any geounit.
unordered_map<string,int> g_THECOUNT;
// end global variables //////////////////////



#include "LUT.h" //for lookup table (for erf)

#define LUTPREC 100000 //precision for lookup table
#define LUTCUTOFF 4 //cutoff value
LUT_function<-LUTCUTOFF*LUTPREC , LUTCUTOFF*LUTPREC, s_double, s_double> LUTerf (erf, 1.0/LUTPREC);

s_double myErf(s_double x){
 if (x<= -LUTCUTOFF)
  return -1;
 else if (x>= LUTCUTOFF)
  return 1;
 else
  return LUTerf(LUTPREC*x);
}




template<class TYPE>
class s_array{
private:
 int my_size;
 TYPE * my_array;
public:


 TYPE at (int k) const {
  return my_array[k];
 }


 s_array(){my_array=nullptr; my_size=0;}
 s_array(int size){
  my_size=size;
  my_array=new TYPE[my_size];
 }

 s_array(const s_array& source){
  if (source.my_array!=nullptr){
    my_size=source.my_size;
    my_array=new TYPE[my_size];
    for (int i=0; i<my_size; i++)
     my_array[i]=source.my_array[i];
   }
  else{
   my_array=nullptr;
   my_size=0;
   }
 }


 s_array(s_array&& source)
 {
  // pilfer other's resource
  my_size=source.my_size;
  my_array=source.my_array;
  source.my_size=0;
  source.my_array=nullptr;
 }

 s_array& operator=(s_array&& source)
 {
  if (&source != this) {
    if (source.my_array!=nullptr){
      delete[] my_array;
      my_array = source.my_array;
      source.my_array = nullptr;
      my_size=source.my_size;
     }
    else{
     my_array=nullptr;
     my_size=0;
     }
   }
  return *this;
 }

 s_array& operator= (const s_array& source){
  if (&source!=this){
    if (source.my_array!=nullptr){
      my_size=source.my_size;
      my_array=new TYPE[my_size];
      for (int i=0; i<my_size; i++)
       my_array[i]=source.my_array[i];
     }
    else{
     my_array=nullptr;
     my_size=0;
     }
   }
  return *this;
 }
 ~s_array(){
  if (my_array!=nullptr)
   delete [] my_array;
 }

 void wipe(){
  if (my_array!=nullptr){
    delete [] my_array;
    my_size=0;
    my_array=nullptr;
   }
 }

 TYPE operator [](int i) const    {
  assert(i>=0 && i<my_size);
  return my_array[i];
 }
 TYPE & operator [](int i) {
  assert(i>=0 && i<my_size);
  return my_array[i];
 }

};



template <class TYPE>
void insertion_sort(s_array<TYPE> & arr, int length) //smallest to largest
{
 int i,j;
 TYPE tmp;
 for (i = 1; i < length; i++) {
   j = i;
   while (j > 0 && arr[j - 1] > arr[j]) {
     tmp = arr[j];
     arr[j] = arr[j - 1];
     arr[j - 1] = tmp;
     j--;
    }
  }
}

template <class TYPE>
void index_insertion_sort(const s_array<TYPE> & array, s_array<int> & index, int length)
{
 int i,j;
 int tmp;
 for (i = 1; i < length; i++) {
   j = i;
   while (j > 0 && array[index[j - 1]] > array[index[j]]) {
     tmp = index[j];
     index[j] = index[j - 1];
     index[j - 1] = tmp;
     j--;
    }
  }
}



template<class TYPE>
class di_array { //doubly indexed array, allows sorted modification

private:
 s_array<int> index;
 s_array<int> rev_index;
 s_array<TYPE> my_array;
 int my_size;

public:

 di_array<TYPE>(){}
 di_array<TYPE>(int size){
  my_size=size;
  my_array=s_array<TYPE>(my_size);
  index=s_array<int>(my_size);
  rev_index=s_array<int>(my_size);
  for (int i=0; i<my_size; i++){
    index[i]=rev_index[i]=i;
    my_array[i]=0;
   }
 }

 TYPE at(int k) const {
  return my_array[k];
 }

 TYPE element_in_order(int k) const {
  return my_array[index[k]];
 }


 void sorted_modify(int k, TYPE value){

  int j;
  int tmp;
  j=rev_index[k];
  assert(index[j]==k);

  if (value <  my_array[k]){
    my_array[k]=value;
    while (j>0 && my_array[index[j-1]] > my_array[index[j]]){
      rev_index[index[j]]--;
      rev_index[index[j - 1]]++;
      tmp = index[j];
      index[j] = index[j - 1];
      index[j - 1] = tmp;
      j--;
     }
   }
  else{
    my_array[k]=value;
    while (j<my_size-1 && my_array[index[j+1]] < my_array[index[j]]){
      rev_index[index[j]]++;
      rev_index[index[j + 1]]--;
      tmp = index[j];
      index[j] = index[j + 1];
      index[j + 1] = tmp;
      j++;
     }
   }
 }
};



inline bool ends_with(std::string const & value, std::string const & ending)
{
 if (ending.size() > value.size()) return false;
 return std::equal(ending.rbegin(), ending.rend(), value.rbegin());
}


struct edge{
 int u;
 int j;
 edge(int a, int b){
  u=a;
  j=b;
 }
 edge(){}
 operator int () {   //assigns an unique integer to an edge
  return (g_MAXDEGREE * (this->u)) + j;
 }
};


bool operator==(const edge& e, const edge& f)
{
 return e.u==f.u && e.j==f.j;
}

bool operator!=(const edge& e, const edge& f)
{
 return e.u!=f.u || e.j!=f.j;
}


ostream & operator << (ostream &out, const edge &e)
{
 out << e.u;
 out << "->" << e.j;
 return out;
}

void Tokenize( const char* szString, vector<std::string>&
               vecstrTokens, const char* szSeparators, bool fNoEmpties){
 const char* pc;
 string strCur;
 bool fPush;

 if( !( pc = szString ) )
  return;

 fPush = false;
 while( true ) {
   strCur.clear( );
   if( fNoEmpties )
    for( ; *pc && strchr( szSeparators, *pc ); ++pc );
   if( !*pc ) {
     if( !fNoEmpties && fPush )
      vecstrTokens.push_back( strCur );
     return; }
   for( ; *pc && !strchr( szSeparators, *pc ); ++pc )
    strCur += *pc;
   if( (fPush = !!*pc) )
    pc++;
   vecstrTokens.push_back( strCur );
  }
}

class geounit{

public:
 int degree;
 s_array<g_int> neighbors; //list of INDICES (in pr[]) of neighbors of the geounit
 s_array<g_int> self; //my place in each of my neighbor's neighbor lists
 s_array<int64_t>  shared_perimeters; //list of shared perimeters with the neighbors
 int64_t area;
 int population;
 int voteA;
 int voteB;
 int original_district;
 int cluster;
 string county;
 uint32_t county_id;
 int precinct;
 int hasInc; //1 if incumbent present, 2 if protected


 int muni_count;
 s_array<uint16_t> muni_index; //list of municipalities intersected
 s_array<s_double> muni_area; //intersection areas with above

 bool frozen;
 bool giant; //Am I in the intial component of the original district.
 friend void computeselves();

 geounit(){
  giant=false;
  frozen=false;
 }

 geounit(int d){ //used for dummy boundary geounit
  degree=d;
  neighbors=s_array<g_int>(degree);
  self=s_array<g_int>(degree);
  shared_perimeters= s_array<int64_t>(degree);
  frozen=false;
  giant=false;
  hasInc=0;
  muni_count=0;
  muni_index=s_array<uint16_t>(0);
  muni_area=s_array<s_double>(0);
 }

 geounit(vector<string> dataline, bool flip, int filetype, bool integerfile){
  //    int myindex=atoi(dataline[0].c_str());
  vector<string> neighbors_string;
  vector<string> shared_perimeters_string;
  int this_index=atoi(dataline[0].c_str());
  Tokenize(dataline[1].c_str(),neighbors_string,",",true);
  Tokenize(dataline[2].c_str(),shared_perimeters_string,",",true);
  degree=neighbors_string.size();
  g_MAXDEGREE=max(degree,g_MAXDEGREE);
  if (shared_perimeters_string.size()!=degree){
    cerr << "ERROR: perimeter list differs in length from neighbor list?" << endl;
    cerr << "neighbor list was: "<< dataline[1] << endl;
    exit(-1);
   }

  neighbors=s_array<g_int>(degree);
  shared_perimeters= s_array<int64_t>(degree);

  for (int i=0; i<degree; i++){
    neighbors[i]=atoi(neighbors_string[i].c_str());
    if (integerfile)
     shared_perimeters[i]=atoi(shared_perimeters_string[i].c_str());
    else{
      b_double this_shared_perimeter=atof(shared_perimeters_string[i].c_str());
      assert(this_shared_perimeter>.00000001);
      assert(this_shared_perimeter<100000);
      shared_perimeters[i]=100000000*this_shared_perimeter;
     }
   }
  if (integerfile)
   area=atoi(dataline[4].c_str());
  else{
    b_double this_area=atof(dataline[4].c_str());
    if (this_area<=.00000001)
     cerr << "WARNING: " << this_index << " has tiny area"<<endl;
    area=ceil(this_area*100000000);
   }

  population=atoi(dataline[5].c_str());
  if (flip){
    voteA=atoi(dataline[7].c_str());
    voteB=atoi(dataline[6].c_str());
   }
  else{
    voteA=atoi(dataline[6].c_str());
    voteB=atoi(dataline[7].c_str());
   }
  if (filetype<=2){
    original_district=atoi(dataline[8].c_str())-1; //subtract one because of indexing from 1 in old files
   }
  else{
    original_district=atoi(dataline[8].c_str());
   }

  if (filetype>1){ //county lines
    county=dataline[9];
    hash<string> hasher;
    county_id=hasher(county);
   }
  if (filetype>2){ //county and muni lines
    vector<string> muniindex_string;
    vector<string> muniarea_string;
    Tokenize(dataline[10].c_str(),muniindex_string,",",true);
    Tokenize(dataline[11].c_str(),muniarea_string,",",true);
    muni_count=muniindex_string.size();
    if (muniarea_string.size()!=muni_count){
      cerr << "ERROR: muni list differs in length from muni intersection area list?" << endl;
      cerr << "muni list was: "<< dataline[10] << endl;
      exit(-1);
     }
    muni_index=s_array<uint16_t>(muni_count);
    muni_area=s_array<s_double>(muni_count);

    for (int i=0; i<muni_count; i++){
      muni_index[i]=atoi(muniindex_string[i].c_str());
      muni_area[i]=atof(muniarea_string[i].c_str());
     }
   }
  else{
    muni_count=0;
    muni_index=s_array<uint16_t>(muni_count);
    muni_area=s_array<s_double>(muni_count);
   }
  if (filetype>3)
   cluster=atoi(dataline[12].c_str());
  else
   cluster=0;

  if (filetype>4){
    precinct=atoi(dataline[13].c_str());
    if (precinct==-1) // no precinct
     precinct=0;
    assert(precinct>=0);
   }

  if (filetype>5){
    int inc_value=atoi(dataline[14].c_str());
    if (!(inc_value==1 || inc_value==0)){
      cerr << "ERROR in incumbent field"<<endl;
      exit(-1);
     }
    hasInc=inc_value;
   }
  else
   hasInc=0;

  frozen=false;
  giant=false;
 }



 int unique_muni(s_double threshold){
  if (muni_count==0)
   return -1;

  int onlymuni=-1;

  for (int i=0; i<muni_count; i++){
    if (muni_area[i] >= threshold){
      if (onlymuni==-1)
       onlymuni=muni_index[i];
      else if (onlymuni!=muni_index[i])
       return -1;
     }
   }

  return max(onlymuni,0);
 }

};


void computeselves(geounit * pr){   //find edges back to me
 //from my neighboring geounitss.
 //where multiple edges are present between a pair,
 //we don't need or ensure geometric correspondence.  Only 1-1 correspondence.

 s_array<s_array<g_int>> boundary = s_array<s_array<g_int>>(g_NUMCLUSTERS);
 s_array<s_array<g_int>> bself    = s_array<s_array<g_int>>(g_NUMCLUSTERS);
 s_array<g_int> boundarydegree = s_array<g_int>(g_NUMCLUSTERS);

 for (int cluster=0; cluster<g_NUMCLUSTERS; cluster++){
   boundary[cluster]=s_array<g_int>(g_NUMGEOS);//indices of geounits bordering boundary, in clockwise order
   bself[cluster]   =s_array<g_int>(g_NUMGEOS); //boundary back pointers
   for (int i=0; i<g_NUMGEOS; i++){
     boundary[cluster][i]=-1;
     bself[cluster][i]   =-1;
    }
   boundarydegree[cluster]=0;
  }



 for (int i=0; i<g_NUMGEOS; i++){
   pr[i].self=s_array<g_int>(pr[i].degree);
   for (int j=0; j<pr[i].degree; j++){
     if (pr[i].neighbors[j]>-1){
       int setcount=0;
       for (int l=0; l<pr[pr[i].neighbors[j]].degree; l++){
         if (pr[pr[i].neighbors[j]].neighbors[l]==i && pr[pr[i].neighbors[j]].shared_perimeters[l]==pr[i].shared_perimeters[j]){
           setcount++;
           pr[i].self[j]=l;
          }
        }
       assert(setcount<2);
       if (setcount!=1){
         cerr << "setcount is " << setcount<<endl;
         cerr << "i is "<< i<<endl;
         cerr << "j is "<< j<<endl;
         cerr << "pr[i].neighbors[j] is " << pr[i].neighbors[j] << endl;
        }
       assert(setcount==1);
      }
     else{
       boundarydegree[pr[i].cluster]++;
       boundary[pr[i].cluster][-pr[i].neighbors[j]-1]=i;
       bself[pr[i].cluster][-pr[i].neighbors[j]-1]=j;

       pr[i].self[j]= (-pr[i].neighbors[j]-1); //outside edges are labeled by order
       pr[i].neighbors[j]=g_NUMGEOS+pr[i].cluster; //reset pointer to boundary geounit
      }
    }
  }


 for (int cluster=0; cluster<g_NUMCLUSTERS; cluster++){
   if (boundarydegree[cluster]==0){
     cerr << "ERROR: Cluster "<<cluster<<" has no neighbors??"<<endl;
     exit(-1);
    }

   assert(boundary[cluster][boundarydegree[cluster]]<0);
   assert(boundary[cluster][boundarydegree[cluster]-1]>=0);

   assert(bself[cluster][boundarydegree[cluster]]<0);
   assert(bself[cluster][boundarydegree[cluster]-1]>=0);

   if (cluster<g_NUMCLUSTERS-1){
    g_MAXDEGREE=max((int) boundarydegree[cluster],g_MAXDEGREE); //ALL BUT LAST MUST SATISFY
    }
   pr[g_NUMGEOS+cluster]=geounit(boundarydegree[cluster]);
   for (int j=0; j<pr[g_NUMGEOS+cluster].degree; j++){
     pr[g_NUMGEOS+cluster].neighbors[j]=boundary[cluster][j];
     pr[g_NUMGEOS+cluster].self[j]=bself[cluster][j];
     pr[g_NUMGEOS+cluster].shared_perimeters[j]=pr[pr[g_NUMGEOS+cluster].neighbors[j]].shared_perimeters[ pr[g_NUMGEOS+cluster].self[j] ];
    }
   pr[g_NUMGEOS+cluster].original_district=-1-cluster; //dummy district
  }

}

//code to check for districts belonging to only one muni
void findsupermunis(geounit * pr, s_array<int> supersetmuni, s_double threshold, int * realdistrict){
 for (int k=0; k<g_NUMDISTRICTS; k++)
  supersetmuni[k]=-2;  //Supermuni possibility initially unknown

 for (int i=0; i<g_NUMGEOS; i++){
   assert(pr[i].muni_count>0);
   if (supersetmuni[pr[i].original_district]!=-1){ //Supermuni not already disproven

     if (pr[i].unique_muni(threshold)==-1)  //no unique muni
      supersetmuni[pr[i].original_district]=-1; //Supermuni is now disproven
     else if (supersetmuni[pr[i].original_district]>=0 &&
              pr[i].unique_muni(threshold) != supersetmuni[pr[i].original_district])
      supersetmuni[pr[i].original_district]=-1; //Supermuni is now disproven
     else
      supersetmuni[pr[i].original_district]=pr[i].unique_muni(threshold); //candidate supermuni
    }
  }

 for (int k=0; k<g_NUMDISTRICTS; k++){
   assert(supersetmuni[k]>-2);
   if (g_debuglevel>0 && supersetmuni[k]!=-1)
    cout << "District "<<realdistrict[k]<<" is a subset of Muni "<<supersetmuni[k] <<endl;
  }
}

void processincumbents(geounit * pr, bool outputlist){
 int num_incumbents=0;
 s_array<int> dist_incumbents(g_NUMDISTRICTS);
 for (int D=0; D<g_NUMDISTRICTS; D++)
  dist_incumbents[D]=0;
 for (int v=0; v<g_NUMGEOS; v++){
   if (pr[v].hasInc){
     assert(pr[v].hasInc==1);
     num_incumbents++;
     dist_incumbents[pr[v].original_district]++;
    }
  }
 for (int v=0; v<g_NUMGEOS; v++){
   if (pr[v].hasInc==1 && dist_incumbents[pr[v].original_district]==1){
     pr[v].frozen=true;
     pr[v].hasInc=2;
    }
  }
 int num_protected=0;
 for (int D=0; D<g_NUMDISTRICTS; D++){
   if (dist_incumbents[D]==1)
    num_protected++;
  }
 if (g_debuglevel>0)
  cout << num_incumbents << " incs, " << num_protected << " prot."<<endl;
}

void freezegeounitsbymuni(geounit * pr,  s_double threshold, int * realdistrict, bool outputlist, bool carecount=false){
 int MAX_MUNI=0;
 for (int v=0; v<g_NUMGEOS; v++){
   for (int j=0; j<pr[v].muni_count; j++)
    MAX_MUNI=max(MAX_MUNI,(int) pr[v].muni_index[j]);
  }
 MAX_MUNI++;
 unordered_map<int,int> munifreeze_map; //unique dists for frozen munis
 s_array<int> munifreeze_count(MAX_MUNI); //geounits for each muni, g_NUMGEOS is upper bound on num of munis
 for (int m=0; m<MAX_MUNI; m++)
  munifreeze_count[m]=0;
 for (int v=0; v<g_NUMGEOS; v++){
   for (int j=0; j<pr[v].muni_count; j++){
     munifreeze_count[pr[v].muni_index[j]]++;
     if (munifreeze_map.count(pr[v].muni_index[j])==0)
      munifreeze_map[pr[v].muni_index[j]]=pr[v].original_district;
     else if (munifreeze_map[pr[v].muni_index[j]]!=pr[v].original_district)
        munifreeze_map[pr[v].muni_index[j]]=-1; //not preserved
    }
  }

 if (outputlist){
   for (auto keyvaluepair : munifreeze_map){
     if (keyvaluepair.second!=-1)
      cout << "Muni "<<keyvaluepair.first << " is contained in district "<< realdistrict[keyvaluepair.second] << " " << munifreeze_count[keyvaluepair.first] << " times" <<endl;
    }
  }

 for (int v=0; v<g_NUMGEOS; v++){
   for (int j=0; j<pr[v].muni_count; j++){
     if (munifreeze_map[pr[v].muni_index[j]]!=-1 && (!carecount || munifreeze_count[pr[v].muni_index[j]]>1)){
       assert(munifreeze_map[pr[v].muni_index[j]]==pr[v].original_district);
       pr[v].frozen=true;
       if (0 && outputlist)
        cout << "preserving muni "<<pr[v].muni_index[j]<<endl;
      }
    }
  }
}

//CODE to check for counties which belong to only one district and set
//their geounits to frozen:
void freezegeounitsbycounty(geounit * pr, bool outputlist){
 unordered_map<string,int> countyfreeze_map; //unique dists for frozen counties
 for (int v=0; v<g_NUMGEOS; v++){
   if (countyfreeze_map.count(pr[v].county)==0)
    countyfreeze_map[pr[v].county]=pr[v].original_district;
   else if (countyfreeze_map[pr[v].county]!=pr[v].original_district)
    countyfreeze_map[pr[v].county]=-1; //not preserved
  }

 if (outputlist){
   for (auto keyvaluepair : countyfreeze_map){
     if (keyvaluepair.second!=-1)
      cout << keyvaluepair.first << " County is preserved by initial districting"<<endl;
    }
  }
 for (int v=0; v<g_NUMGEOS; v++){
   if (countyfreeze_map[pr[v].county]!=-1){
     assert(countyfreeze_map[pr[v].county]==pr[v].original_district);
     pr[v].frozen=true;
    }
   else
    assert(pr[v].frozen==false);
  }
}


template<class TYPE>  //type must be uniquely castable to (int)
class simp_set{ //keeps items in first mycount pos'n of array
protected:
 int max; //max number of TYPES to be stored
 int maxindex; //max size of (int) TYPE to be seen

 s_array<int> index; //index of locations

 //for single level
 s_array<TYPE> array;
 int mycount; //number of elements


public:

 int count(){
  return mycount;
 }

 bool empty(){
  return (mycount==0);
 }

 TYPE at(int i){
  assert (i>=0 && i<mycount);
  return array[i];
 }

 bool contains(TYPE e){
  return (index[(int) e]>=0);
 }

 TYPE& access(TYPE e){ //returns the copy that I contain
  return array[index[(int) e]];
 }

 int check(){
  int thiscount=0;
  for (int i=0; i<maxindex; i++){
    if (index[i]>-1){
      thiscount++;
      assert(thiscount==mycount);
     }
   }
  return thiscount;
 }

 void insert(TYPE e){
  assert(index[(int) e]==-1);
  index[(int) e]=mycount;
  array[mycount++]=e;
  assert( index[(int) e]==mycount-1 );
 }
 void remove(TYPE e){
  int i=index[(int) e];
  assert (i>=0 && i<mycount);
  array[i]=array[--mycount];
  index[(int) array[i]]=i;
  index[(int) e]=-1;
  assert(mycount>=0);
 }

 simp_set<TYPE>(){
 }

 simp_set<TYPE>(int m, int M){
  max=m;
  maxindex=M;
  array=s_array<TYPE>(max);
  index=s_array<int>(maxindex);
  for (int i=0; i<maxindex; i++)
   index[i]=-1; //means: nowhere
  mycount=0;
 }

};


template<class TYPE>
class ndup_set : public simp_set<TYPE> { //allows random access

public:
 bool nondup_insert(TYPE e){
  if (this->index[(int) e]==-1){
    this->insert(e);
    return true;
   }
  else{
    return false;
   }
 }

 ndup_set<TYPE>() : simp_set<TYPE>(){}
 ndup_set<TYPE>(int m, int M) : simp_set<TYPE>(m, M){}
};


template<class TYPE>
class rnd_set : public simp_set<TYPE> { //allows random access

public:
 TYPE& randomaccess(mt19937_64& gen){
  uniform_int_distribution<int> intdist(0,this->mycount-1);
  return this->array[intdist(gen)];
 }

 rnd_set<TYPE>() : simp_set<TYPE>(){}
 rnd_set<TYPE>(int m, int M) : simp_set<TYPE>(m, M){}
 rnd_set<TYPE> (const simp_set<TYPE>& source) : simp_set<TYPE>(source){}

};


struct proposal{
 int gunit;
 int district;

 proposal(){}
 proposal(int g,int d){
  gunit=g;
  district=d;
 }

 operator int () {
  return g_NUMDISTRICTS*this->gunit+district;
 }
};


bool operator==(const proposal& propA, const proposal& propB)
{
 return propA.gunit==propB.gunit && propA.district==propB.district;
}



class proposalpile{
private:
protected:

public:
 proposalpile(){
 }
 virtual bool propose(proposal myprop, int j)=0; //true if myprop is new
 virtual bool remove_proposal_reason(proposal myprop, int j)=0; //true if proposal dies
 virtual proposal random(mt19937_64& gen)=0;
 virtual int count()=0;
 virtual bool ugly_precinct(int p)=0;
 virtual ~proposalpile(){};
 virtual proposalpile* Clone()=0;
 virtual void set_thresh()=0;
 virtual void set_thresh(int thresh)=0;
 virtual int broken_precincts()=0;
 virtual bool nice()=0;

};



class uni_proposalpile : public proposalpile{
private:
 s_array< ndup_set<g_int> > proposal_reasons;
 rnd_set<proposal> alive_proposals;

public:
 uni_proposalpile* Clone()
 {
  return new uni_proposalpile(*this);
 }

 virtual bool nice(){return true;}

 virtual void set_thresh(){}
 virtual void set_thresh(int thresh){}

 virtual int broken_precincts(){return 0;}

 virtual bool ugly_precinct(int p){return false;}

 virtual bool propose(proposal myprop, int j){
  if (  proposal_reasons[(int) myprop].nondup_insert(j) && proposal_reasons[(int) myprop].count()==1){
    alive_proposals.insert(myprop);
    return true;
   }
  return false;
 }

 virtual bool remove_proposal_reason(proposal myprop, int j){
  proposal_reasons[(int) myprop].remove(j);
  if (proposal_reasons[(int) myprop].count()==0){
    alive_proposals.remove(myprop);
    return true;
   }
  return false;
 }

 virtual proposal random(mt19937_64& gen){
  proposal myprop=alive_proposals.randomaccess(gen);
  return myprop;
 }

 virtual int count(){
  return alive_proposals.count();
 }

 virtual proposal proposal_at(int i){
  return alive_proposals.at(i);
 }

 uni_proposalpile(){
 }

 uni_proposalpile(geounit * pr){

  alive_proposals=rnd_set<proposal>(g_NUMGEOS*g_NUMDISTRICTS, g_NUMGEOS*g_NUMDISTRICTS);

  proposal_reasons=s_array<ndup_set<g_int>>(g_NUMGEOS*g_NUMDISTRICTS); //max proposal int cast
  for (int v=0; v<g_NUMGEOS; v++){
    for (int D=0; D<g_NUMDISTRICTS; D++){
      proposal_reasons[(int) proposal(v,D)]=ndup_set<g_int>(pr[v].degree,pr[v].degree);
     }
   }
 }
};


class dub_proposalpile : public proposalpile{
private:
 float cut_slack;
 s_array<g_int> sizeof_precinct; //num geounits in each precinct
 s_array<b_double> perimof_precinct; //perimeter of each precinct
 s_array<g_int> order_in_precinct; //compress geo id to consec. within precinct
 s_array<b_double> Dperim_in_precinct; //length of boundary crossing precinct

 geounit * pr;

 s_array<ndup_set<edge>> bound_edges_in_precinct; //district boundary edges within prec.
 s_array<bool> precinct_broken;                   //is precinct broken

 int num_broken_precincts;

 int thresh_broken_precincts;

 uni_proposalpile all_proposals;
 int num_precincts;

 s_array<int> proposals_in_precinct;
 int within_broken_precinct_proposals;

public:
 dub_proposalpile* Clone()
 {
  return new dub_proposalpile(*this);
 }

 virtual int broken_precincts(){return num_broken_precincts;}

 bool nice(){
  return num_broken_precincts<thresh_broken_precincts;
 }

 void set_thresh(){
  thresh_broken_precincts=num_broken_precincts+1;
  cout << "--Splitting thresh set at "<<    thresh_broken_precincts<<endl;
 }

 void set_thresh(int thresh){
  if (thresh<num_broken_precincts){
    cerr << "ERROR: precinct breaking threshold or exceeded when set"<<endl;
    exit(-1);
   }
  thresh_broken_precincts=thresh+1;
  cout << "--Splitting thresh set at "<<    thresh_broken_precincts<<endl;
 }



 virtual bool propose(proposal myprop, int j){
  bool newprop=all_proposals.propose(myprop,j);
  if (newprop)
   proposals_in_precinct[pr[myprop.gunit].precinct]++;

  if ( pr[pr[myprop.gunit].neighbors[j]].precinct == pr[myprop.gunit].precinct && //new edge
       bound_edges_in_precinct[pr[myprop.gunit].precinct].nondup_insert(edge(order_in_precinct[myprop.gunit],j))){
    Dperim_in_precinct[pr[myprop.gunit].precinct]+=pr[myprop.gunit].shared_perimeters[j];
   if (bound_edges_in_precinct[pr[myprop.gunit].precinct].count()==1  ){  //becomes broken here
     assert(precinct_broken[pr[myprop.gunit].precinct]==false);
     precinct_broken[pr[myprop.gunit].precinct]=true;
     num_broken_precincts++;
     within_broken_precinct_proposals+=proposals_in_precinct[pr[myprop.gunit].precinct];
    }
   else if (newprop && precinct_broken[pr[myprop.gunit].precinct]==true) //already broken
    within_broken_precinct_proposals++;
   }
  else if (newprop && precinct_broken[pr[myprop.gunit].precinct]==true) //already broken
   within_broken_precinct_proposals++;

  return newprop;
 }

 virtual bool remove_proposal_reason(proposal myprop, int j){
  bool deadprop=all_proposals.remove_proposal_reason(myprop,j);
  if (pr[pr[myprop.gunit].neighbors[j]].precinct == pr[myprop.gunit].precinct){ //same precinct
    bound_edges_in_precinct[pr[myprop.gunit].precinct].remove(edge(order_in_precinct[myprop.gunit],j));
    Dperim_in_precinct[pr[myprop.gunit].precinct]-=pr[myprop.gunit].shared_perimeters[j];
    if ( bound_edges_in_precinct[pr[myprop.gunit].precinct].count()==0 ){  //newly unbroken
      assert(precinct_broken[pr[myprop.gunit].precinct]==true);
      assert(Dperim_in_precinct[pr[myprop.gunit].precinct]<.0000001); //0 up to double precision losses
      Dperim_in_precinct[pr[myprop.gunit].precinct]=0;
      precinct_broken[pr[myprop.gunit].precinct]=false;
      num_broken_precincts--;
      within_broken_precinct_proposals-=proposals_in_precinct[pr[myprop.gunit].precinct];
     }
   }
  if (deadprop && precinct_broken[pr[myprop.gunit].precinct]==true) //still broken
   within_broken_precinct_proposals--;

  if (deadprop)
   proposals_in_precinct[pr[myprop.gunit].precinct]--;

  assert(num_broken_precincts>=0);


  assert(num_broken_precincts<=thresh_broken_precincts);

  return deadprop;
 }

 virtual int num_edges_in_precinct(int p){
  return bound_edges_in_precinct[p].count();
 }

 virtual int geos_in_precinct(int p){
  return sizeof_precinct[p];
 }

 virtual bool ugly_precinct(int p){
  //return (num_edges_in_precinct(p)*num_edges_in_precinct(p) > 40*geos_in_precinct(p)+10);
  return Dperim_in_precinct[p]>cut_slack * perimof_precinct[p];
 }

 virtual proposal random(mt19937_64& gen){
  assert(count()>0);
  if (num_broken_precincts<thresh_broken_precincts)
   return all_proposals.random(gen);
  else{
    proposal myprop=all_proposals.random(gen);
    while (! precinct_broken[pr[myprop.gunit].precinct] )
     myprop=all_proposals.random(gen);
    return myprop;
   }
 }


 virtual int count(){
  assert(within_broken_precinct_proposals>=0);
  assert(within_broken_precinct_proposals<=all_proposals.count());

  if (num_broken_precincts<thresh_broken_precincts){
    assert(all_proposals.count()>0);
    return all_proposals.count();
   }
  else{
    //cerr << "within...proposals="<<within_broken_precinct_proposals<<" and numbrokenprecincts="<<num_broken_precincts<<endl;
   return within_broken_precinct_proposals;
   }
 }




 dub_proposalpile(){
 }

 dub_proposalpile(geounit * thepr, b_double Cut_Slack){
  cut_slack=Cut_Slack;
  pr=thepr;
  within_broken_precinct_proposals=0;

  order_in_precinct=s_array<g_int>(g_NUMGEOS);

  //count precincts:
  int maxprecinct=0;
  for (int v=0; v<g_NUMGEOS; v++)
   maxprecinct=max(maxprecinct,pr[v].precinct);

  num_precincts=maxprecinct+1;

  perimof_precinct=s_array<b_double>(num_precincts);
  sizeof_precinct=s_array<g_int>(num_precincts);
  for (int p=0; p<num_precincts; p++){
   sizeof_precinct[p]=0;
   perimof_precinct[p]=0;
   }
  for (int v=0; v<g_NUMGEOS; v++){
   order_in_precinct[v]=(sizeof_precinct[pr[v].precinct]++);
   for (int j=0; j<pr[v].degree; j++){
     if (pr[pr[v].neighbors[j]].precinct != pr[v].precinct)
      perimof_precinct[pr[v].precinct]+=pr[v].shared_perimeters[j];
    }
   }
  for (int p=0; p<num_precincts; p++){
    if (sizeof_precinct[p]==0 && g_debuglevel>0)
     cerr << "precinct " << p << " is empty?"<<endl;
   }

  Dperim_in_precinct=s_array<b_double>(num_precincts);
  precinct_broken=s_array<bool>(num_precincts);
  proposals_in_precinct=s_array<int>(num_precincts);
  bound_edges_in_precinct=s_array<ndup_set<edge>>(num_precincts);
  for (int p=0; p<num_precincts; p++){
    bound_edges_in_precinct[p]=ndup_set<edge>(sizeof_precinct[p]*g_MAXDEGREE,sizeof_precinct[p]*g_MAXDEGREE);
    precinct_broken[p]=false;
    proposals_in_precinct[p]=0;
    Dperim_in_precinct[p]=0;
   }

  num_broken_precincts=thresh_broken_precincts=0;
  all_proposals=uni_proposalpile(pr);
 }
};




b_double dcompact(int64_t const & Area, int64_t const & Perim){  //inverse of Polsby-Popper: bigger is worse
 assert(Area>0);
 return ((b_double) Perim*Perim)/(((b_double) Area));//*4*M_PI);
}

b_double PopperDelta(const s_array<int64_t> &areas, s_array<int64_t> &perims, b_double thresh){
 b_double excess=1.;
 for (int D=0; D<g_NUMDISTRICTS; D++)
  excess*=max((b_double) 1.,   dcompact(areas[D], perims[D])  / thresh );
 return excess;
}

b_double compactsum(const s_array<int64_t> &Area, s_array<int64_t> &Perim, int arraylength){
 b_double sum=0;
 for (int i=0; i<arraylength; i++)
  sum+=dcompact(Area[i],Perim[i]);
 return sum;
}


b_double compactL2(const s_array<int64_t> &Area, s_array<int64_t> &Perim, int arraylength){
 b_double sum=0;
 for (int i=0; i<arraylength; i++)
  sum+=pow(dcompact(Area[i],Perim[i]),2);
 return sum;
}

b_double compactLh(const s_array<int64_t> &Area, s_array<int64_t> &Perim, int arraylength){
 b_double sum=0;
 for (int i=0; i<arraylength; i++)
  sum+=pow(dcompact(Area[i],Perim[i]),.5);
 return sum;
}

template<class TYPE>
TYPE arraysum(const s_array<TYPE> &array, int arraylength){
 b_double sum=0;
 for (int i=0; i<arraylength; i++)
  sum+=array[i];
 return sum;
}

template<class TYPE>
TYPE arrayL2(const s_array<TYPE> &array, int arraylength){
 TYPE sum=0;
 for (int i=0; i<arraylength; i++)
  sum+=pow(array[i],2);
 return sum;
}

int reps(const di_array<s_double> &Ashare, s_double winthresh){
 int reps=0;
 for (int k=0; k<g_NUMDISTRICTS; k++){
   if (Ashare.at(k)>winthresh)
    reps++;
  }
 return reps;
}

s_double variance(const di_array<s_double> &Ashare)
{
 s_double meansq=0;
 s_double mean=0;
 for (int k=0; k<g_NUMDISTRICTS; k++){
   meansq+=pow(Ashare.at(k),2);
   mean+=Ashare.at(k);
  }
 meansq=meansq/g_NUMDISTRICTS;
 mean=mean/g_NUMDISTRICTS;
 return meansq-pow(mean,2);
}


s_double efficiency_gap(const s_array<int> & DvotesA, const s_array<int> & DvotesB)
{
 int total=0;
 for (int k=0; k<g_NUMDISTRICTS; k++){
   total+=DvotesA[k];
   total+=DvotesB[k];
  }
 int wastedA=0;
 int wastedB=0;
 for (int k=0; k<g_NUMDISTRICTS; k++){
   if (DvotesA[k]>DvotesB[k]){
     wastedA+=(DvotesA[k]-DvotesB[k])/2;
     wastedB+=DvotesB[k];
    }
   else{
     wastedB+=(DvotesB[k]-DvotesA[k])/2;
     wastedA+=DvotesA[k];
    }
  }
 return ((s_double) wastedA-wastedB)/total;
}

s_double median_mean(const di_array<s_double> & Ashare)
{
 s_double mean,median,sum;
 int middle=(g_NUMDISTRICTS+1)/2; //top median
 if (!(g_NUMDISTRICTS%2))//for median with even g_NUMDISTRICTS
  median=(Ashare.element_in_order(middle)+Ashare.element_in_order(middle-1))/2;
 else  //median for odd g_NUMDISTRICTS
  median=Ashare.element_in_order(middle-1);
 sum=0;
 for (int k=0; k<g_NUMDISTRICTS; k++)
  sum+=Ashare.at(k);
 mean=sum/g_NUMDISTRICTS;
 return mean-median;
}



s_double seat_thresh(const di_array<s_double> & Ashare, int maxseats)
{
 return .5- Ashare.element_in_order(g_NUMDISTRICTS-maxseats-1);
}

s_double seat_slide(const di_array<s_double> &Ashare, s_double slideSD, s_double slideMu)
{
 s_double seatslide=0;
 for (int k=0; k<g_NUMDISTRICTS; k++)
  seatslide+=.5*(1+myErf( (Ashare.at(k)-slideMu)/(sqrt(2)*slideSD) ));
 //seatslide+=   min( max( Ashare.at(k)-.45 , 0. )/.1 , 1.);//UNIFORM


 return seatslide;
}


struct metric{
 s_double variance;
 s_double median_mean;
 s_double seat_slide;
 s_double seat_thresh;
 s_double efficiency_gap;
 int seat_count;

 metric(s_double V, s_double M, s_double SS, s_double ST, s_double EG, int S){
  variance=V;
  median_mean=M;
  seat_slide=SS;
  seat_thresh=ST;
  efficiency_gap=EG;
  seat_count=S;
 }
 metric(){}
};

#define LARGEPRIME 9223372036854775783 //2^63-25

bool contains_power_of_two(int64_t n, int k){
 return (n & (~(1<<k))) == 0;
}


template <class myClass>
class clone_ptr
{
public:
 clone_ptr(){location=nullptr;}
 clone_ptr(myClass* d) { location = d;}
 ~clone_ptr() { delete location; }

 clone_ptr(const clone_ptr<myClass>& source){
  if (source.location!=nullptr){
    location=source.location->Clone();
   }
  else
   location=nullptr;
 }
 clone_ptr& operator= (const clone_ptr<myClass>& source){
  if (&source!=this){
    if (source.location!=nullptr){
      location=source.location->Clone();
     }
    else
     location=nullptr;
   }
  return *this;
 }

 myClass* operator->() { return location; }
 myClass& operator*() { return *location; }

private:
 myClass* location;
};


struct Ddata{
private:
 s_array<D_int> currentdistrict;

public:

 int  net_traversal_count;

 s_array<bool> frozen_districts;
 s_array<bool> has_protected_incumbent;

 geounit * pr; //just a pointer
 s_array<int> votesA;
 s_array<int> votesB;
 s_array<int> pops;
 di_array<s_double> Ashare;
 s_array<int64_t> areas;
 s_array<int64_t> perims;

 int64_t perimeter;
 b_double L1compactness;
 b_double L2compactness;
 b_double Lhcompactness;


 s_double avgpop;

 clone_ptr<proposalpile> swapproposals;

 b_double score; //used for MH;

 bool multilevel;

 edge reverse(edge e){
  return edge(pr[e.u].neighbors[e.j],pr[e.u].self[e.j]);
 }


 edge nextinface(edge e){
  return edge(pr[e.u].neighbors[e.j], (pr[e.u].self[e.j]-1+pr[pr[e.u].neighbors[e.j]].degree) % pr[pr[e.u].neighbors[e.j]].degree);
 }

 edge nextaroundvertex(edge e){
  return edge(e.u, e.j<pr[e.u].degree-1 ? e.j+1 : 0);
 }

 edge previnface(edge e){
  return reverse(nextaroundvertex(e));
 }

 int tail(edge e){
  return e.u;
 }

 int head(edge e){
  return pr[e.u].neighbors[e.j];
 }

 bool is_countyboundary(edge e){
  return (pr[tail(e)].county_id!=pr[head(e)].county_id && tail(e)<g_NUMGEOS && head(e)<g_NUMGEOS);
 }

 int current_district(int i){
  return (int) currentdistrict[i];
 }

 int county_id(int i){
  return pr[i].county_id;
 }

 void set_current_district(int i, int16_t d){
  currentdistrict[i]=d;
 }

 string curr_districting_as_string(){
  string districting_string="";
  for (int v=0; v<g_NUMGEOS; v++){
    districting_string+=to_string(current_district(v));
    districting_string+=",";
   }
  return districting_string;
 }

 void reset_L1compactness(){
  L1compactness=compactsum(areas,perims,g_NUMDISTRICTS);
 }
 void reset_L2compactness(){
  L2compactness=compactL2(areas,perims,g_NUMDISTRICTS);
 }
 void reset_Lhcompactness(){
  Lhcompactness=compactLh(areas,perims,g_NUMDISTRICTS);
 }
 bool reset_perimeter(){
  int64_t new_perimeter=arraysum(perims,g_NUMDISTRICTS);
  if (perimeter!=new_perimeter){
   perimeter=new_perimeter;
   return true;
   }
  return false;
 }

 Ddata(){
 }

 Ddata(geounit * PR, s_array<bool> froz_dists, bool multi, b_double cut_slack=1){
  pr=PR;
  frozen_districts=froz_dists;
  multilevel=multi;

  votesA= s_array<int>(g_NUMDISTRICTS);      //running count of voteA-voteB in each district
  votesB= s_array<int>(g_NUMDISTRICTS);      //running count of voteA-voteB in each district
  Ashare= di_array<s_double>(g_NUMDISTRICTS);
  pops=   s_array<int>(g_NUMDISTRICTS);      //running count of voteA-voteB in each district
  areas=  s_array<int64_t>(g_NUMDISTRICTS);   //district areas
  perims= s_array<int64_t>(g_NUMDISTRICTS);  //district perimeters
  has_protected_incumbent=s_array<bool>(g_NUMDISTRICTS);

  for (int k=0; k<g_NUMDISTRICTS; k++){
    votesA[k]=votesB[k]=pops[k]=0;
    areas[k]=perims[k]=0;
    has_protected_incumbent[k]=false;
   }

  for (int v=0; v<g_NUMGEOS; v++){
    if (pr[v].hasInc==2)
     has_protected_incumbent[pr[v].original_district]=true;
   }

  score=1.;

  net_traversal_count=0;

  currentdistrict=s_array<D_int>(g_NUMGEOS+g_NUMCLUSTERS);
  for (int i=0; i<g_NUMGEOS+g_NUMCLUSTERS; i++)
   currentdistrict[i]=pr[i].original_district;

  if (!multilevel)
   swapproposals=clone_ptr<proposalpile>(new uni_proposalpile(pr));
  else{
   swapproposals=clone_ptr<proposalpile>(new dub_proposalpile(pr, cut_slack));
   }
  for (int i=0; i<g_NUMGEOS; i++){
    for (int j=0; j<pr[i].degree; j++){
      int idistrict=pr[i].original_district;
      int jdistrict=pr[pr[i].neighbors[j]].original_district;
      if (pr[i].neighbors[j]<g_NUMGEOS){  //not a boundary connection
        if (idistrict!=jdistrict &&
            !frozen_districts[idistrict] &&
            !frozen_districts[jdistrict] &&
            !pr[pr[i].neighbors[j]].frozen
            ){
          swapproposals->propose(proposal(pr[i].neighbors[j], idistrict), pr[i].self[j]);
         }
       }
     }
   }

  s_array<bool> seen(g_NUMDISTRICTS);
  for (int i=0; i<g_NUMDISTRICTS; i++){
    pops[i]=votesA[i]=votesB[i]=areas[i]=0;
    seen[i]=false;
   }

  int totalpop=0;

  for (int i=0; i<g_NUMGEOS; i++){
    seen[current_district(i)]=true;
    pops[current_district(i)]+=pr[i].population;
    totalpop+=pr[i].population;
    votesA[current_district(i)]+= pr[i].voteA;
    votesB[current_district(i)]+= pr[i].voteB;
    areas[current_district(i)]+=  pr[i].area;
    for (int j=0; j<pr[i].degree; j++){
      if (pr[i].neighbors[j]>=0 && current_district(i)!=current_district(pr[i].neighbors[j]))
       perims[current_district(i)]+=pr[i].shared_perimeters[j];
      else if (pr[i].neighbors[j]<0)
       perims[current_district(i)]+=pr[i].shared_perimeters[j];
     }
   }
  for (int p=0; p<g_NUMPRECINCTS; p++){
    if (swapproposals->ugly_precinct(p)){
      cerr << "ERROR: precinct "<<p<< " is cut badly"<<endl;
      exit(-1);
     }
   }
  avgpop = (s_double) totalpop/g_NUMDISTRICTS;
  for (int i=0; i<g_NUMDISTRICTS; i++){
    if (seen[i]==false){
      cerr << "ERROR: some districts are empty?  Perhaps change -N value."<<endl;
      exit(-1);
      assert(areas[i]>0);
      assert(perims[i]>0);
     }
   }


  for (int k=0; k<g_NUMDISTRICTS; k++){
    Ashare.sorted_modify(k,((s_double) votesA[k])/(votesA[k]+votesB[k]));
   }

  if (multilevel)
   swapproposals->set_thresh(); //set precinct breaking threshold

  reset_L1compactness();
  reset_L2compactness();
  reset_Lhcompactness();
  reset_perimeter();
 }
};

struct condbag{
 bool Perim;
 bool L1;
 bool L2;
 bool Lh;
 bool Popper;

 bool Traversals;

 int64_t perimthresh;
 b_double L1thresh;
 b_double L2thresh;
 b_double Lhthresh;
 b_double popperthresh;

 int upperpop,lowerpop;


 bool MH;
 b_double cLambda;
 b_double pLambda;
 b_double offLambda; //offset

 bool validpop(int population) const {
  return population>=lowerpop && population<=upperpop;
 }


 b_double popDelta(s_array<int> & pops) const {
  b_double excess=1.;
  for (int D=0; D<g_NUMDISTRICTS; D++)
   excess*=max((b_double) 1.,    max( (b_double) pops[D]/upperpop , (b_double) lowerpop/pops[D] ));
  return excess;
 }



 condbag(gengetopt_args_info lineArgs, Ddata firstD, int * realdistrict){
  Perim=lineArgs.perimeter_given;
  L1= lineArgs.L1_compactness_given;
  L2= lineArgs.L2_compactness_given;
  Lh= lineArgs.Lh_compactness_given;

  Popper=lineArgs.polsby_popper_given;

  if ((lineArgs.perimeter_given && lineArgs.perimeter_arg<1) || (lineArgs.L1_compactness_given && lineArgs.L1_compactness_arg<1) || (lineArgs.L2_compactness_given && lineArgs.L2_compactness_arg<1) || (lineArgs.polsby_popper_given && lineArgs.polsby_popper_arg<1)){
    cerr << "ERROR: threshold must be above 1 (100%)"<<endl;
    exit(-1);
   }


  MH=(lineArgs.compact_lambda_given || lineArgs.pop_lambda_given);
  cLambda=lineArgs.compact_lambda_arg;
  pLambda=lineArgs.pop_lambda_arg;
  offLambda=lineArgs.off_lambda_arg;

  Traversals=lineArgs.county_traversals_flag;


  if (lineArgs.upperpop_given || lineArgs.lowerpop_given){
    if (!lineArgs.upperpop_given || !lineArgs.lowerpop_given || lineArgs.poperror_given){
      cerr << "ERROR: give either both of upperpop and lowerpop, or neither, and omit poperror when you give them"<<endl;
      exit(-1);
     }
    upperpop=lineArgs.upperpop_arg;
    lowerpop=lineArgs.lowerpop_arg;
   }
  else
   {
    upperpop=firstD.avgpop*(1+lineArgs.poperror_arg);
    lowerpop=ceil(firstD.avgpop*(1-lineArgs.poperror_arg));
   }

  for (int i=0; i<g_NUMDISTRICTS; i++){
    //cout << "district " <<realdistrict[i]<<" has population "<<firstD.pops[i]<<endl;
    if (firstD.pops[i]<lowerpop || firstD.pops[i]>upperpop){
      cerr << "ERROR: district "<<realdistrict[i]<<" violates population threshold."<<endl<<"Limits are "<<lowerpop<<"," <<upperpop<<" and its pop is "<<firstD.pops[i]<<endl;
      exit(-1);
     }
   }

  if (Perim){
    int64_t currentperim=arraysum(firstD.perims,g_NUMDISTRICTS);
    perimthresh=currentperim*lineArgs.perimeter_arg;
   }
  if (Popper){
    b_double currentpopper=0;
    for (int k=0; k<g_NUMDISTRICTS; k++){
      currentpopper=max(currentpopper,dcompact(firstD.areas[k],firstD.perims[k]));
     }
    popperthresh=lineArgs.polsby_popper_arg*currentpopper;
   }
  if (L1){
    b_double currentl1=compactsum(firstD.areas,firstD.perims,g_NUMDISTRICTS);
    L1thresh=currentl1*lineArgs.L1_compactness_arg;
   }
  if (L2){
    b_double currentl2=compactL2(firstD.areas,firstD.perims,g_NUMDISTRICTS);
    L2thresh=currentl2*lineArgs.L2_compactness_arg;
   }
  if (Lh){
    b_double currentlh=compactLh(firstD.areas,firstD.perims,g_NUMDISTRICTS);
    Lhthresh=currentlh*lineArgs.Lh_compactness_arg;
   }

 }
};


void tosvg(char* svgfilename, char* inputsvgfilename, Ddata & myD, int firstline, int highlight=-1)
{
 ofstream districtingsvg(svgfilename);
 ifstream inputsvg(inputsvgfilename);
 if (!districtingsvg.good() || !inputsvg.good()){
   cerr << "ERROR with one of the svg files"<<endl;
   exit(-1);
  }
 string correct_header ("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");


 mt19937_64 gen(314159265358979); //reinitialize so colors are consistent on each run
 std::uniform_int_distribution<int> randpercent(0,100);
 s_array<string> color=s_array<string>(g_NUMDISTRICTS);
 s_array<string> highlightcolor=s_array<string>(g_NUMDISTRICTS);
 for (int i=0; i<g_NUMDISTRICTS; i++){
   int red=randpercent(gen);
   int green=randpercent(gen);
   int blue=randpercent(gen);
   char tmpcolor[100];
   sprintf(tmpcolor,"rgb(%d%%,%d%%,%d%%);",red,blue,green);
   color[i]=string(tmpcolor);
   sprintf(tmpcolor,"rgb(%d%%,%d%%,%d%%);",100-(100-red)/2,100-(100-blue)/2,100-(100-green)/2);
   highlightcolor[i]=string(tmpcolor);
  }
 string line;
 getline (inputsvg,line); //file header
 districtingsvg << line << endl;
 if (correct_header.compare(line)){
   cerr << "ERROR: incorrect file header at pos " << correct_header.compare(line) << endl;
   exit(-1);
  }

 for (int i=0; i<firstline-2; i++){
   getline (inputsvg,line);
   districtingsvg << line << endl;
  }

 for (int v=0; v<g_NUMGEOS; v++){
   assert(myD.current_district(v)>=0);
   assert(myD.current_district(v)<g_NUMDISTRICTS);
   getline (inputsvg,line); //color line ignored
   bool extrahighlight=false;
   // EXAMPLE
   //if (v==318)// || myD.pr[v].precinct==19 || myD.pr[v].precinct==29 || myD.pr[v].precinct==109)
    //extrahighlight=true;
   if (extrahighlight || v==highlight)
    districtingsvg << highlightcolor[myD.current_district(v)] << endl;
   else
    districtingsvg << color[myD.current_district(v)]<<endl;

   getline (inputsvg,line);
   districtingsvg << line << endl;
  }
 while (getline (inputsvg,line))
  districtingsvg << line;
 districtingsvg.close();
 inputsvg.close();
}

void tofile(char* filename, Ddata & myD,  int filetype)
{
 ofstream output(filename);
 if (!output.good()){
   cerr << "ERROR with geounit output file"<<endl;
   exit(-1);
  }
 if (filetype==6)
  output << "precinctlistv06int" <<endl;
 else if (filetype==5)
  output << "precinctlistv05int" <<endl;
 else if (filetype==4)
  output << "precinctlistv04int" <<endl;
 else if (filetype==3)
  output << "precinctlistv03int" <<endl;
 else if (filetype==2)
  output << "precinctlistv02int" <<endl;
 else{
   assert(filetype==1);
   output << "precinctlistv01int" <<endl;
  }
 output << g_NUMGEOS << endl;
 output << "\tnb\tsp\tunshared\tarea\tpop\tvoteA\tvoteB\tcongD";
 if (filetype>=2)
  output << "\tcounty";
 if (filetype>=3)
  output << "\tmuniIndex\tmuniArea";
 if (filetype>=4)
  output << "\tClusterID";
 if (filetype>=5)
  output << "\tprecinct";
 if (filetype>=6)
  output << "\thasInc";
 output <<endl;
 for (int i=0; i<g_NUMGEOS; i++){
   output << i;

   output << '\t';

   for (int j=0; j<myD.pr[i].degree-1; j++){
     if (myD.pr[i].neighbors[j]<g_NUMGEOS)
      output << myD.pr[i].neighbors[j] << ',';
     else
      output << (-myD.pr[i].self[j]-1) << ',';
    }
   if (myD.pr[i].neighbors[myD.pr[i].degree-1]<g_NUMGEOS)
    output << myD.pr[i].neighbors[myD.pr[i].degree-1];
   else
    output << (-myD.pr[i].self[myD.pr[i].degree-1]-1);

   output << '\t';

   for (int j=0; j<myD.pr[i].degree-1; j++)
    output << myD.pr[i].shared_perimeters[j] << ',';
   output << myD.pr[i].shared_perimeters[myD.pr[i].degree-1];
   output << '\t';
   output << 0;
   output << '\t';
   output << myD.pr[i].area;
   output << '\t';
   output << myD.pr[i].population;
   output << '\t';
   output << myD.pr[i].voteA;
   output << '\t';
   output << myD.pr[i].voteB;
   output << '\t';
   if (filetype<=2)
    output << myD.current_district(i)+1;
   else
    output << myD.current_district(i);
   if (filetype>=2){
     output << '\t';
     output << myD.pr[i].county;
    }
   if (filetype>=3){
     for (int k=0; k<myD.pr[i].muni_count-1; k++)
      output << myD.pr[i].muni_index[k]<<',';
     output << myD.pr[i].muni_index[myD.pr[i].muni_count-1]<<'\t';
     for (int k=0; k<myD.pr[i].muni_count-1; k++)
      output << myD.pr[i].muni_area[k]<<',';
     output << myD.pr[i].muni_area[myD.pr[i].muni_count-1];
    }
   if (filetype>=4)
    output << '\t'<<myD.pr[i].cluster;
   if (filetype>=5)
    output << '\t'<<myD.pr[i].precinct;
   if (filetype>=6)
    output << '\t'<<myD.pr[i].hasInc;
   output << endl;
  }
 output.close();
}

void tosvg(Ddata & myD, gengetopt_args_info lineArgs, int t, int outputcount){
 if (lineArgs.svg_filename_given){
   char svgfilename[100];
   sprintf(svgfilename,"%s_%d_%.3d.svg",lineArgs.svg_filename_arg,t,outputcount);
   char inputsvgfilename[100];
   sprintf(inputsvgfilename,"%s.svg",lineArgs.filename_arg);
   tosvg(svgfilename,inputsvgfilename,myD,lineArgs.svg_firstline_arg);
  }
}

class datacount{
public:
 int64_t variance_lessunusual;
 int64_t median_mean_lessunusual;
 int64_t seat_slide_lessunusual;
 int64_t seat_thresh_lessunusual;
 int64_t efficiency_gap_lessunusual;
 int64_t seat_count_lessunusual;

 int64_t variance_moreunusual;
 int64_t median_mean_moreunusual;
 int64_t seat_slide_moreunusual;
 int64_t seat_thresh_moreunusual;
 int64_t efficiency_gap_moreunusual;
 int64_t seat_count_moreunusual;

 int64_t totalvalidsteps;
 int64_t totalbridgesteps;

 s_array<int64_t> myreps_histogram;



 datacount(int firstmore){
  variance_lessunusual=0;
  median_mean_lessunusual=0;
  seat_slide_lessunusual=0;
  seat_thresh_lessunusual=0;
  efficiency_gap_lessunusual=0;
  seat_count_lessunusual=0;

  assert(firstmore!=0);

  variance_moreunusual=firstmore;
  median_mean_moreunusual=firstmore;
  seat_slide_moreunusual=firstmore;
  seat_thresh_moreunusual=firstmore;
  efficiency_gap_moreunusual=firstmore;
  seat_count_moreunusual=firstmore;

  totalvalidsteps=firstmore;
  totalbridgesteps=0;
  myreps_histogram=s_array<int64_t>(g_NUMDISTRICTS+1);
  for (int i=0; i<=g_NUMDISTRICTS; i++)
   myreps_histogram[i]=0;


 }

 datacount(){
  variance_lessunusual=0;
  median_mean_lessunusual=0;
  seat_slide_lessunusual=0;
  seat_thresh_lessunusual=0;
  efficiency_gap_lessunusual=0;
  seat_count_lessunusual=0;

  variance_moreunusual=0;
  median_mean_moreunusual=0;
  seat_slide_moreunusual=0;
  seat_thresh_moreunusual=0;
  efficiency_gap_moreunusual=0;
  seat_count_moreunusual=0;

  totalvalidsteps=0;
  totalbridgesteps=0;

  myreps_histogram=s_array<int64_t>(g_NUMDISTRICTS+1);
  for (int i=0; i<=g_NUMDISTRICTS; i++)
   myreps_histogram[i]=0;
 }

 s_double variance_ep(){
  return (s_double) variance_moreunusual/(variance_moreunusual+variance_lessunusual);
 }

 s_double median_mean_ep(){
  return (s_double) median_mean_moreunusual/(median_mean_moreunusual+median_mean_lessunusual);
 }

 s_double seat_slide_ep(){
  return (s_double) seat_slide_moreunusual/(seat_slide_moreunusual+seat_slide_lessunusual);
 }

 s_double seat_thresh_ep(){
  return (s_double) seat_thresh_moreunusual/(seat_thresh_moreunusual+seat_thresh_lessunusual);
 }

 s_double efficiency_gap_ep(){
  return (s_double) efficiency_gap_moreunusual/(efficiency_gap_moreunusual+efficiency_gap_lessunusual);
 }

 s_double seat_count_ep(){
  return (s_double) seat_count_moreunusual/(seat_count_moreunusual+seat_count_lessunusual);
 }

 datacount& operator+= ( datacount otherdata ){
  variance_lessunusual      +=otherdata.variance_lessunusual;
  median_mean_lessunusual   +=otherdata.median_mean_lessunusual;
  seat_slide_lessunusual    +=otherdata.seat_slide_lessunusual;
  seat_thresh_lessunusual   +=otherdata.seat_thresh_lessunusual;
  efficiency_gap_lessunusual+=otherdata.efficiency_gap_lessunusual;
  seat_count_lessunusual    +=otherdata.seat_count_lessunusual;

  variance_moreunusual      +=otherdata.variance_moreunusual;
  median_mean_moreunusual   +=otherdata.median_mean_moreunusual;
  seat_slide_moreunusual    +=otherdata.seat_slide_moreunusual;
  seat_thresh_moreunusual   +=otherdata.seat_thresh_moreunusual;
  efficiency_gap_moreunusual+=otherdata.efficiency_gap_moreunusual;
  seat_count_moreunusual    +=otherdata.seat_count_moreunusual;

  totalvalidsteps           +=otherdata.totalvalidsteps;
  totalbridgesteps          +=otherdata.totalbridgesteps;

  for (int D=0; D<=g_NUMDISTRICTS; D++)
   myreps_histogram[D]+=otherdata.myreps_histogram[D];
  return *this;
 }

};





class binom{
 s_array< s_array<int64_t> > binomial_array;
 int N;
public:
 binom(){
 }
 binom(int N){
  this->N=N;
  binomial_array=s_array<s_array<int64_t>>(N+1);
  for (int n=0; n<=N; n++){
    binomial_array[n]=s_array<int64_t>(n+1);
    binomial_array[n][0]=1;
    binomial_array[n][n]=1;
    for (int k=1; k<n; k++){
     if (binomial_array[n-1][k-1] <= numeric_limits<int64_t>::max() / 2 &&
         binomial_array[n-1][k]   <= numeric_limits<int64_t>::max() / 2)
      binomial_array[n][k]=binomial_array[n-1][k-1]+binomial_array[n-1][k];
     else
      binomial_array[n][k]=-1;
     }
   }
 }
 int64_t choose(int n,int k) const{
  return binomial_array[n][k];
 }
};

b_double binomial_upper_tail(int n, int K, b_double p, const binom& threadbinom){
 b_double tailprobability=0;
 for (int k=K; k<=n; k++){
   if (threadbinom.choose(n,k)==-1)
    return 1; //binomial overflow
   else
    tailprobability+= threadbinom.choose(n,k)*pow(p,k)*pow(1-p,n-k);
  }
 return tailprobability;
}

template <class TYPE>
string signif_format(TYPE f, int n)
{
 if (f == 0) {
   return "0";
  }
 int d = (int)::floor(::log10(f < 0 ? -f : f))+1; /*digits before decimal point*/
 TYPE order = ::pow(10., n - d);
 std::stringstream ss;
 ss << std::fixed << std::setprecision(std::max(n - d, 0)) << round(f * order) / order;
 return ss.str();
}

struct outputbag{
 bool doVariance;
 bool doMedianMean;
 bool doSeatSlide;
 bool doSeatThresh;
 bool doEfficiencyGap;
 bool doSeats;
 bool doHistogram;
 bool doGraph;
 s_double slideSD;
 s_double slideMu;
 s_double winthresh;
 int maxseats;

 metric initial;

 s_array<bool>  thread_alive;





 s_double avgpop;


 stringstream epss_to_output(const s_array<s_double> & my_epss, int T, bool doubling_rhos, int num_rhos, s_double first_alpha_mult) const {
  stringstream output;
  for (int t=0; t<T; t++){
    output << setw(5)<< my_epss[t]<<" ";
   }
  output << endl;
  s_array<s_double> sorted_epss=my_epss;
  insertion_sort(sorted_epss,T);
  binom threadbinom=binom(T+1);
  int rhodiff=0;
  for (int rhocount=0; rhocount<num_rhos; rhocount++){
    int rho=T-rhodiff;
    if (rho<1)
     break;
    s_double eps=sorted_epss[rho-1];
    bool firstforthisrho=true;
    for (s_double alphamult=first_alpha_mult; alphamult<=24; alphamult*=2){
      s_double alpha=eps*alphamult;
      s_double pvalue=binomial_upper_tail(T,rho,sqrt(2*eps/alpha),threadbinom);
      if (pvalue<.1){
        if (firstforthisrho){
          output << "rho="<<rho<<":"<<endl;
          firstforthisrho=false;
         }
        if (eps<.000002 || alpha<.000002)
         output << scientific<<setprecision(2);
        else
         output << fixed << left<< setprecision(6);

        output << "we are an     ("<< eps<<", "<<alpha<<")    outlier     at  p=" << signif_format(pvalue,2)<< endl;
       }
     }
    if (!doubling_rhos || rhodiff==0)
     rhodiff++;
    else if (rhodiff<T/2)
     rhodiff*=2;
    else
     rhodiff=(rhodiff+T)/2;
   }
  return output;
 }

public:

 outputbag (gengetopt_args_info lineArgs, Ddata firstD, mt19937_64 gen, int * realdistrict){
  avgpop=firstD.avgpop;
  doVariance=lineArgs.variance_flag;
  doMedianMean=lineArgs.median_mean_flag;
  doSeatSlide=lineArgs.seat_slide_flag;
  doSeatThresh=lineArgs.seat_thresh_flag;
  doEfficiencyGap=lineArgs.efficiency_gap_flag;
  doSeats=lineArgs.seats_flag;
  doHistogram=lineArgs.histogram_flag;
  doGraph=lineArgs.graph_flag;
  slideSD=lineArgs.slideSD_arg;
  slideMu=lineArgs.slideMu_arg;
  winthresh=lineArgs.win_thresh_arg;
  maxseats=lineArgs.maxseats_arg;
  if (maxseats>=g_NUMDISTRICTS){
    cerr << "Not sensible to set --maxseats to the number of districts or greater"<<endl;
    exit(-1);
   }

  initial=metric(variance(firstD.Ashare), median_mean(firstD.Ashare), seat_slide(firstD.Ashare,slideSD,slideMu), seat_thresh(firstD.Ashare,maxseats), efficiency_gap(firstD.votesA,firstD.votesB), reps(firstD.Ashare,winthresh));




  //bool intermediate_output=(! lineArgs.only_end_flag);





  if (! lineArgs.only_end_flag){
    if (doVariance){
      cout << "initial variance is "<<initial.variance<<endl;
     }
    if (doMedianMean){
      cout << "initial median_mean is "<<initial.median_mean<<endl;
     }
    if (doSeatSlide){
      cout << "initial seat_slide is "<<initial.seat_slide<<endl;
     }
    if (doSeatThresh){
      cout << "initial seat_thresh is "<<initial.seat_thresh<<endl;
     }
    if (doEfficiencyGap){
      cout << "initial efficiency_gap is "<<initial.efficiency_gap<<endl;
     }
    if (doSeats){
      cout << "initial seat count is "<<initial.seat_count<<endl;
     }
    if (doGraph){
      for (int j=0; j<g_NUMDISTRICTS; j++)
       cout << "D"<<setw(3) << realdistrict[j] << ": "<<firstD.Ashare.at(j)<<endl;
      cout << endl;
     }
   }
 }


 void update(datacount & mydata, Ddata & myD, int revisitations, int64_t STEPS) const { //update counts for thread t

  if (!myD.swapproposals->nice() || myD.score>1.){
    mydata.totalbridgesteps+=revisitations;
    return;
   }

  mydata.totalvalidsteps+=revisitations;

  //    if (mydata.totalsteps%10)
  //      return;

  if (doVariance){
    if (variance(myD.Ashare)>=initial.variance)
     mydata.variance_moreunusual+=revisitations;
    else
     mydata.variance_lessunusual+=revisitations;
   }
  if (doMedianMean){
    if (median_mean(myD.Ashare)>=initial.median_mean)
     mydata.median_mean_moreunusual+=revisitations;
    else
     mydata.median_mean_lessunusual+=revisitations;
   }
  if (doSeatSlide){
    if (seat_slide(myD.Ashare,slideSD,slideMu)<=initial.seat_slide)
     mydata.seat_slide_moreunusual+=revisitations;
    else
     mydata.seat_slide_lessunusual+=revisitations;
   }
  if (doSeatThresh){
    if (seat_thresh(myD.Ashare,maxseats)>=initial.seat_thresh)
     mydata.seat_thresh_moreunusual+=revisitations;
    else
     mydata.seat_thresh_lessunusual+=revisitations;
   }
  if (doEfficiencyGap){
    if (efficiency_gap(myD.votesA,myD.votesB)>=initial.efficiency_gap)
     mydata.efficiency_gap_moreunusual+=revisitations;
    else
     mydata.efficiency_gap_lessunusual+=revisitations;
   }
  if (doSeats){
    if (reps(myD.Ashare,winthresh)<=initial.seat_count)
     mydata.seat_count_moreunusual+=revisitations;
    else
     mydata.seat_count_lessunusual+=revisitations;
   }
  if (doHistogram){
    mydata.myreps_histogram[reps(myD.Ashare,winthresh)]+=revisitations;
   }


 }

 s_double pvalue(s_double epsilon) const {
  if (StatTest==rootep)
   return sqrt(2*epsilon);
  else if (StatTest==twoep)
   return 2*epsilon;
  else if (StatTest==outlier)
   return 2*epsilon;
  else if (StatTest==starsplit)
   return epsilon;
  else{
    cerr << "ERROR: what StatTest?"<<endl;
    exit(-1);
   }
  return 1;
 }


 void output(s_array<datacount> & threadcounts, Ddata & myD, int T) const {
  stringstream output;
  datacount mydata;
  for (int t=0; t<T; t++)
    mydata+=threadcounts[t];

  cout << forscreen(mydata, myD).str();
 }

 void outlier_output(s_array<datacount> & threadcounts, int T, bool doubling_rhos, int num_rhos, s_double alpha_mult) const {
  assert(StatTest==outlier);


  stringstream output;

  if (doVariance){
    s_array<s_double> variance_epss(T);
    for (int t=0; t<T; t++)
     variance_epss[t]=threadcounts[t].variance_ep();
    output << "--FOR VARIANCE--"<<endl;
    output << epss_to_output(variance_epss, T, doubling_rhos, num_rhos, alpha_mult).str();
   }
  if (doMedianMean){
    s_array<s_double> median_mean_epss(T);
    for (int t=0; t<T; t++)
     median_mean_epss[t]=threadcounts[t].median_mean_ep();
    output << "--FOR MEDIAN MEAN--"<<endl;
    output << epss_to_output(median_mean_epss, T, doubling_rhos, num_rhos, alpha_mult).str();
   }
  if (doSeatSlide){
    s_array<s_double> seat_slide_epss(T);
    for (int t=0; t<T; t++){
      seat_slide_epss[t]=threadcounts[t].seat_slide_ep();
     }
    output << "--FOR SEAT SLIDE--"<<endl;
    output << epss_to_output(seat_slide_epss, T, doubling_rhos, num_rhos, alpha_mult).str();
   }
  if (doSeatThresh){
    s_array<s_double> seat_thresh_epss(T);
    for (int t=0; t<T; t++)
     seat_thresh_epss[t]=threadcounts[t].seat_thresh_ep();
    output << "--FOR SEAT THRESH--"<<endl;
    output << epss_to_output(seat_thresh_epss, T, doubling_rhos, num_rhos, alpha_mult).str();
   }
  if (doEfficiencyGap){
    s_array<s_double> efficiency_gap_epss(T);
    for (int t=0; t<T; t++)
     efficiency_gap_epss[t]=threadcounts[t].efficiency_gap_ep();
    output << "--FOR EFFICIENCY GAP--"<<endl;
    output << epss_to_output(efficiency_gap_epss, T, doubling_rhos, num_rhos, alpha_mult).str();
   }
  if (doSeats){
    s_array<s_double> seats_epss(T);
    for (int t=0; t<T; t++)
     seats_epss[t]=threadcounts[t].seat_count_ep();
    output << "--FOR SEATS--"<<endl;
    output << epss_to_output(seats_epss, T, doubling_rhos, num_rhos, alpha_mult).str();
   }
  output << endl;
  cout << output.str();
 }





 stringstream forscreen(datacount mydata, Ddata & myD) const {

  stringstream output;

  output << (s_double) mydata.totalvalidsteps/(mydata.totalvalidsteps+mydata.totalbridgesteps)<<" dist ratio"<<endl;
  s_double maxpopdev=0;
  for (int D=0; D<g_NUMDISTRICTS; D++)
   maxpopdev=max(maxpopdev,    max( (s_double) myD.pops[D]/avgpop-1, 1-(s_double) myD.pops[D]/avgpop ) );
  output << "max pop deviation is "<<maxpopdev<<endl;


  if (g_debuglevel>0){
    output << "Dshare is ";
    for (int j=0; j<g_NUMDISTRICTS; j++)
     output << setprecision(2) << 100*myD.Ashare.at(j) <<"%, ";
    output << endl;
   }





  if (doHistogram){
    for (int j=0; j<=g_NUMDISTRICTS; j++){
      if (mydata.myreps_histogram[j]>0){
        output << setw(2) << j << ": ";
        for (int k=0; k<(50*mydata.myreps_histogram[j])/(mydata.totalvalidsteps); k++)
         output << "*";
        for (int k=ceil(50*mydata.myreps_histogram[j])/(mydata.totalvalidsteps); k<50; k++)
         output << " ";
        output <<100*(s_double) mydata.myreps_histogram[j]/(mydata.totalvalidsteps)<<"%"<<endl;
       }
     }
    output << endl;
   }

  if (doGraph && g_NUMDISTRICTS<20){
    for (int j=0; j<g_NUMDISTRICTS; j++)
     output << "D"<<setw(2) << j << ": "<<myD.Ashare.at(j)<<endl;
    output << endl;
   }
  if (doVariance){
    output << "--FOR VARIANCE--"<<endl;
    output << "current variance is "<<setprecision(5)<< variance(myD.Ashare)<<endl;
    output << "thread moreunusual is " << mydata.variance_moreunusual<<endl;
    output << "thread lessunusual is " << mydata.variance_lessunusual<<endl;

    s_double V_ep=mydata.variance_ep();
    output << "ep="<<V_ep<<endl;
    output << "p="<<pvalue(V_ep)<<endl << endl;
   }
  if (doMedianMean){
    output << "--FOR MEDIAN/MEAN--"<<endl;
    output << "current median_mean is "<< setprecision(5)<< median_mean(myD.Ashare) << endl;
    output << "thread moreunusual is " << mydata.median_mean_moreunusual<<endl;
    output << "thread lessunusual is " << mydata.median_mean_lessunusual<<endl;

    s_double M_ep=mydata.median_mean_ep();
    output << "ep="<<M_ep<<endl;
    output << "p="<<pvalue(M_ep)<<endl << endl;
   }
  if (doSeatSlide){
    output << "--FOR SEAT/SLIDE--"<<endl;
    output << "current seat_slide is "<< setprecision(5)<< seat_slide(myD.Ashare,slideSD,slideMu) << endl;
    output << "thread moreunusual is " << mydata.seat_slide_moreunusual<<endl;
    output << "thread lessunusual is " << mydata.seat_slide_lessunusual<<endl;

    s_double SS_ep=mydata.seat_slide_ep();
    output << "ep="<<SS_ep<<endl;
    output << "p="<<pvalue(SS_ep)<<endl << endl;
   }


  if (doSeatThresh){
    output << "--FOR SEAT_THRESH--"<<endl;
    output << "current seat_thresh is "<< setprecision(5)<< seat_thresh(myD.Ashare,maxseats) << endl;
    output << "thread moreunusual is " << mydata.seat_thresh_moreunusual<<endl;
    output << "thread lessunusual is " << mydata.seat_thresh_lessunusual<<endl;
    s_double ST_ep=mydata.seat_thresh_ep();
    output << "ep="<<ST_ep<<endl;
    output << "p="<<pvalue(ST_ep)<<endl << endl;
   }




  if (doEfficiencyGap){
    output << "--FOR Efficiency Gap--"<<endl;
    output << "current efficiency_gap is "<< setprecision(5)<< efficiency_gap(myD.votesA,myD.votesB) << endl;
    output << "thread moreunusual is " << mydata.efficiency_gap_moreunusual<<endl;
    output << "thread lessunusual is " << mydata.efficiency_gap_lessunusual<<endl;
    s_double EG_ep=mydata.efficiency_gap_ep();
    output << "ep="<<EG_ep<<endl;
    output << "p="<<pvalue(EG_ep)<<endl << endl;
   }
  if (doSeats){
    output << "--FOR Seat Count--"<<endl;
    output << "current seat_count is "<< setw(2)<< reps(myD.Ashare,winthresh) << endl;
    output << "thread moreunusual is " << mydata.seat_count_moreunusual<<endl;
    output << "thread lessunusual is " << mydata.seat_count_lessunusual<<endl;
    s_double S_ep=mydata.seat_count_ep();
    output << "ep="<<S_ep<<endl;
    output << "p="<<pvalue(S_ep)<<endl << endl;
   }
  return output;
 }


};


bool incumbent_check(Ddata & myD, const proposal myprop){
 return (  (!myD.has_protected_incumbent[myprop.district]) || (myD.pr[myprop.gunit].hasInc==0)  );
}

bool connectivity_check(Ddata & myD, const proposal myprop){
 const int Du=myprop.district;
 const int v=myprop.gunit; //we cyclically visit edges around v.
 const int Dv=myD.current_district(v);

 int Dusegmentcount=0;
 int Dvsegmentcount=0;

 int testcount=0;

 edge f=edge(v,0);
 edge lastincycle=myD.previnface(myD.reverse(f));
 while (f!=lastincycle){
   testcount++;
   assert(testcount<200);

   f=myD.nextinface(f);

   if (myD.pr[f.u].neighbors[f.j]==v){ //if I'm a neighbor of v, skip v for next step of cycle
     f.j>0 ? f.j-- : f.j=myD.pr[f.u].degree-1;   //now f=nextinface(reverse(f,pr),pr)
    }

   int lastD=myD.current_district(f.u);
   int newD=myD.current_district(myD.pr[f.u].neighbors[f.j]);
   if (lastD==Du && newD!=Du)
    Dusegmentcount++;
   else if (lastD==Dv && newD!=Dv)
    Dvsegmentcount++;
  }

 //  if (Dvsegmentcount==0 || Dusegmentcount==0){
 if (Dusegmentcount==0){
   cerr << "Dusegmentcount error" <<endl;

   char svgfilename[100];
   sprintf(svgfilename,"conn_error.svg");
   char inputsvgfilename[100];
   sprintf(inputsvgfilename,"for_error.svg");
   tosvg(svgfilename,inputsvgfilename,myD,4,v);

  }

 assert(Dusegmentcount>0);
 //  assert(Dvsegmentcount>0);

 if (Dvsegmentcount==0){
   return false;
   cerr << "Dvsegmentcount WARNING: encountered singleton?"<<endl;
  }

 if (Dusegmentcount>1 || Dvsegmentcount>1)
  return false;
 else
  return true;
}

void segment_check(Ddata & myD, int D, edge f, int & Dcountyconnections){
 Dcountyconnections++;  //another potential connection to my county in Dv
 if ( (myD.county_id(myD.head(myD.nextaroundvertex(f))) != myD.county_id(myD.head(f)) ||
       (myD.current_district(myD.head(myD.nextaroundvertex(f))) != D)))
  return;  //definitely an new segment;

 edge lastinthisface=myD.reverse(myD.nextaroundvertex(f));
 int testcount=0;
 while (!(f==lastinthisface || myD.is_countyboundary(f) || myD.current_district(myD.head(f))!=D) ){
   testcount++;
   assert(testcount<200);
   f=myD.nextinface(f);
  }
 if (f==lastinthisface)
  Dcountyconnections--;  //same as next connection;
 return;
}


int traversal_change(Ddata & myD, proposal myprop){
 int Du=myprop.district;
 int v=myprop.gunit;
 int Dv=myD.current_district(v);

 int minDvconnections=0;
 int minDuconnections=0;
 int Dvcountyconnections=0;
 int Ducountyconnections=0;
 for (int j=0; j<myD.pr[v].degree; j++){
   if (! myD.is_countyboundary(edge(v,j))){
     if (myD.current_district(myD.head(edge(v,j)))==Dv) {
       edge f=edge(v,j);
       minDvconnections=1;
       segment_check(myD,Dv,f,Dvcountyconnections);
      }
     else if (myD.current_district(myD.head(edge(v,j)))==Du) {
       edge f=edge(v,j);
       minDuconnections=1;
       segment_check(myD,Du,f,Ducountyconnections);
      }
    }
  }
 Dvcountyconnections=max(minDvconnections,Dvcountyconnections);
 Ducountyconnections=max(minDuconnections,Ducountyconnections);


 return (1-Ducountyconnections)-(1-Dvcountyconnections);
}

bool shouldloop(Ddata &myD, const condbag & Test, int v, int Du, int Dv, b_double newDuPP, b_double newDvPP, bool DEBUG){
 if (!Test.validpop(myD.pops[Du]) || !Test.validpop(myD.pops[Dv])){
   if (DEBUG)
    cout << "LOOP! for population violation"<<endl;
   return true;
  }
 else if (Test.Perim && myD.perimeter>Test.perimthresh){
   if (DEBUG)
    cout << "LOOP! for Perimeter violation"<<endl;
   return true;
  }
 else if (Test.Popper){
   if (newDuPP>Test.popperthresh || newDvPP >Test.popperthresh){
     if (DEBUG)
      cout << "LOOP! for Polsby-Popper violation"<<endl;
     return true;
    }
  }
 else if (Test.L1 && myD.L1compactness>Test.L1thresh){
   if (DEBUG)
    cout << "LOOP! for compactness L1 violation"<<endl;
   return true;
  }
 else if (Test.L2 && myD.L2compactness>Test.L2thresh){
   if (DEBUG)
    cout << "LOOP! for compactness L2 violation"<<endl;
   return true;
  }
 else if (Test.Lh && myD.Lhcompactness>Test.Lhthresh){
   if (DEBUG)
    cout << "LOOP! for compactness Lh violation"<<endl;
   return true;
  }
 else if (myD.multilevel && myD.swapproposals->ugly_precinct(myD.pr[v].precinct)){
   if (DEBUG)
    cout << "LOOP! for precinct cut violation"<<endl;
   return true;
  }
 return false;
}



void chainstep(proposal myprop, Ddata &myD, const condbag & Test, bool Verify, mt19937_64 & gen){  //add e.u(e.j) to district of e.u


 int trav_change; //county traversals change
 if (Verify && Test.Traversals){
  trav_change=traversal_change(myD,myprop);
  if (myD.net_traversal_count+trav_change>0) //illegal move because of county traversals
   return;
  }
 else
  trav_change=0;


 geounit * pr=myD.pr;
 int Du=myprop.district;
 int v=myprop.gunit;

 assert(!pr[v].frozen);
 int Dv=myD.current_district(v);
 if (Du==Dv){
   cout <<"WTF! Du is "<<Du<<" and Dv is "<<Dv<<endl;
   exit(-1);
  }
 assert (Dv>=0); //not adding outside
 assert (Du!=Dv);


 b_double oldDuPP=-1;
 b_double oldDvPP=-1;
 int64_t oldDuperim=-1;
 int64_t oldDvperim=-1;
 if (Test.L1 || Test.L2 || Test.Lh){
   oldDuPP=dcompact(myD.areas[Du],myD.perims[Du]);
   oldDvPP=dcompact(myD.areas[Dv],myD.perims[Dv]);
 }
 if (Test.Perim){
   oldDuperim=myD.perims[Du];
   oldDvperim=myD.perims[Dv];
  }


 assert(!myD.frozen_districts[Du]);
 assert(!myD.frozen_districts[Dv]);

 myD.votesA[Du]+= pr[v].voteA;
 myD.votesA[Dv]-= pr[v].voteA;
 myD.votesB[Du]+= pr[v].voteB;
 myD.votesB[Dv]-= pr[v].voteB;
 myD.Ashare.sorted_modify(Du, ((s_double) myD.votesA[Du])/(myD.votesA[Du]+myD.votesB[Du]) );
 myD.Ashare.sorted_modify(Dv, ((s_double) myD.votesA[Dv])/(myD.votesA[Dv]+myD.votesB[Dv]) );

 myD.pops[Du]+= pr[v].population;
 myD.pops[Dv]-= pr[v].population;

 myD.areas[Du]+= pr[v].area;
 myD.areas[Dv]-= pr[v].area;

 for (int l=0; l<pr[v].degree; l++){
   if (pr[v].neighbors[l]>=g_NUMGEOS){  //outside
     myD.perims[Du]+=pr[v].shared_perimeters[l]; //because it avoids Du
     myD.perims[Dv]-=pr[v].shared_perimeters[l]; //because it avoids Dv
    }
   else if (myD.current_district(pr[v].neighbors[l])==Du){
     if (!pr[pr[v].neighbors[l]].frozen)
      myD.swapproposals->remove_proposal_reason(proposal(pr[v].neighbors[l],Dv),pr[v].self[l]);

     myD.swapproposals->remove_proposal_reason(proposal(v,Du),l);
     myD.perims[Du]-=pr[v].shared_perimeters[l]; //because it intersects Du
     myD.perims[Dv]-=pr[v].shared_perimeters[l]; //because it avoids Dv

    }
   else if (myD.current_district(pr[v].neighbors[l])==Dv){
     if (!pr[pr[v].neighbors[l]].frozen)
      myD.swapproposals->propose(proposal(pr[v].neighbors[l],Du),pr[v].self[l]);

     myD.swapproposals->propose(proposal(v,Dv),l);
     myD.perims[Du]+=pr[v].shared_perimeters[l]; //because it avoids Du
     myD.perims[Dv]+=pr[v].shared_perimeters[l]; //because it intersects Dv
    }
   else if (!myD.frozen_districts[ myD.current_district(pr[v].neighbors[l]) ]){ //avoids Du AND Dv, not frozen
     if (!pr[pr[v].neighbors[l]].frozen){
       myD.swapproposals->remove_proposal_reason(proposal(pr[v].neighbors[l],Dv),pr[v].self[l]);
       myD.swapproposals->propose(proposal(pr[v].neighbors[l],Du),pr[v].self[l]);
      }
     myD.perims[Du]+=pr[v].shared_perimeters[l]; //because it avoids Du
     myD.perims[Dv]-=pr[v].shared_perimeters[l]; //because it avoids Dv
    }
   else{ //a frozen district neighbor
     myD.perims[Du]+=pr[v].shared_perimeters[l]; //because it avoids Du
     myD.perims[Dv]-=pr[v].shared_perimeters[l]; //because it avoids Dv
    }
  }


 myD.set_current_district(v,Du);

 b_double newDuPP=dcompact(myD.areas[Du],myD.perims[Du]);
 b_double newDvPP=dcompact(myD.areas[Dv],myD.perims[Dv]);
 if (Test.L1)
  myD.L1compactness=myD.L1compactness-oldDuPP-oldDvPP+newDuPP+newDvPP;
 if (Test.L2)
  myD.L2compactness=myD.L2compactness-oldDuPP*oldDuPP-oldDvPP*oldDvPP+newDuPP*newDuPP+newDvPP*newDvPP;
 if (Test.Lh)
  myD.Lhcompactness=myD.Lhcompactness-pow(oldDuPP,.5)-pow(oldDvPP,.5)+pow(newDuPP,.5)+pow(newDvPP,.5);
 if (Test.Perim)
  myD.perimeter=myD.perimeter-oldDuperim-oldDvperim+myD.perims[Du]+myD.perims[Dv];




 bool LOOP=false;
 if (Verify && !Test.MH){     //CHECK whether we have to REVERSE the move (hard)
   LOOP=shouldloop(myD, Test, v, Du, Dv, newDuPP, newDvPP, (g_debuglevel>1));
  }
 else if (Verify && Test.MH){     //CHECK whether we have to REVERSE the move (soft)
   b_double popscore=Test.popDelta(myD.pops);
   b_double compscore=1.;
   if (myD.multilevel && myD.swapproposals->ugly_precinct(pr[v].precinct)){
     LOOP=true;
     if (g_debuglevel>1)
      cout << "LOOP! for precinct cut violation"<<endl;
    }
   else{
     if (Test.Perim)
      compscore*=max((b_double) 1.,(b_double) myD.perimeter/ (b_double) Test.perimthresh);
     if (Test.Popper)
      compscore*=PopperDelta(myD.areas, myD.perims, Test.popperthresh);
     if (Test.L1)
      compscore*=max((b_double) 1.,myD.L1compactness/Test.L1thresh);
     if (Test.L2)
      compscore*=max((b_double) 1.,myD.L2compactness/Test.L2thresh);
     if (Test.Lh)
      compscore*=max((b_double) 1.,myD.Lhcompactness/Test.Lhthresh);


     b_double new_score=pow(popscore,Test.pLambda)*pow(compscore,Test.cLambda);
     //    if (new_score>0)
     //      new_score+=Test.offLambda;
     if (new_score<=myD.score)       //we definitely accept
      myD.score=new_score;
     else{
       bernoulli_distribution flip(myD.score/new_score);
       if (flip(gen)) //accept the move
        myD.score=new_score;
       else{
         LOOP=true; //reverse it, score stays the same.
        }
      }
    }
  }


 if (LOOP){
   chainstep(proposal(v,Dv), myD, Test, false, gen);
  }
 else {
   myD.net_traversal_count+=trav_change;
  }
 return;
}


bool supermuni_check(Ddata & myD, proposal myprop, const s_array<int> & supersetmuni, s_double threshold){

 int Du=myprop.district;
 int v=myprop.gunit;

 if (supersetmuni[Du]<=0 || supersetmuni[Du]==myD.pr[v].unique_muni(threshold))
  return true;
 else
  return false;
}


void MainLoop(Ddata &myD, const outputbag & Out, datacount & mydata, const condbag & Tests, const s_array<int> & supersetmuni, const s_double threshold,
              const gengetopt_args_info lineArgs, const int filetype, const int64_t STEPS, const int t, mt19937_64 & gen, int & outputcount, int64_t & step_pos, int64_t & my_bigsteps, int64_t output_period){
 //bool intermediate_output=(! lineArgs.only_end_flag);
 int steplength_exp=lineArgs.steplength_arg;
 int64_t steplength=pow(2,steplength_exp);  //Markov Chain exponent

 while (mydata.totalvalidsteps<STEPS){
   while (step_pos<steplength){
     do {
       if (myD.swapproposals->count()==0){
         cerr << "ERROR: no proposals possible, exiting at step_pos="<<to_string(step_pos)<<endl;
         exit(-1);
        }
       proposal myprop=myD.swapproposals->random(gen);    //we'll try adding vertex u(j) to u's district

       if (connectivity_check(myD, myprop)){
         if (supermuni_check(myD, myprop, supersetmuni, threshold)){
           if (!lineArgs.incumbents_flag || incumbent_check(myD,myprop)){
             chainstep(myprop, myD, Tests, true, gen);
            }
          }
        }
       s_double p=((s_double) myD.swapproposals->count())/g_MAXEDGES;
       assert(p>0);
       std::geometric_distribution<int64_t> revisits(p);
       int64_t revisitations=1+revisits(gen);
       step_pos+=revisitations;
      } while (!myD.swapproposals->nice());
   }
   int64_t steps_taken=step_pos/steplength;
   step_pos=step_pos % steplength;

   my_bigsteps++;
   if (contains_power_of_two(my_bigsteps,max(20-steplength_exp,1))){ //iframe to reset any accumlated floating point errors
     if (Tests.L1)
      myD.reset_L1compactness();
     if (Tests.L2)
      myD.reset_L2compactness();
     if (Tests.Lh)
      myD.reset_Lhcompactness();
     if (Tests.Perim)
      if (myD.reset_perimeter()){
       cerr << "Perimeter ERROR"<<endl;
       exit(-1);
       }
    }


   if (lineArgs.testsuite_flag){
     string curr_districting=myD.curr_districting_as_string();
     if (g_THECOUNT.count(curr_districting)==0){
       g_THECOUNT[curr_districting]=steps_taken;
      }
     else
      g_THECOUNT[curr_districting]+=steps_taken;
    }
   Out.update(mydata, myD, steps_taken, STEPS); // update counts for this step


   if (output_period>0){
     if ( (mydata.totalvalidsteps/output_period > (mydata.totalvalidsteps-steps_taken)/output_period) ||  mydata.totalvalidsteps>=STEPS ){
       outputcount++;
       cout << "Thread "<< t<< " reporting. " << endl << Out.forscreen(mydata, myD).str()<<endl;
       if (lineArgs.stages_flag && !(StatTest==outlier)){
         if (lineArgs.svg_filename_given){
           char svgfilename[100];
           sprintf(svgfilename,"%s_%d_%.3d.svg",lineArgs.svg_filename_arg,t,outputcount);
           char inputsvgfilename[100];
           sprintf(inputsvgfilename,"%s.svg",lineArgs.filename_arg);
           tosvg(svgfilename,inputsvgfilename,myD,lineArgs.svg_firstline_arg);
          }

         if (lineArgs.geounit_filename_given){
           char geounitfilename[100];
           sprintf(geounitfilename,"%s_%d_%.3d",lineArgs.geounit_filename_arg,t,outputcount);
           tofile(geounitfilename,myD,filetype);
          }
        }
       if (lineArgs.svg_filename_given && t==0){
         tosvg(myD, lineArgs, t, outputcount);
        }

       if (lineArgs.geounit_filename_given){
         char geounitfilename[100];
         sprintf(geounitfilename,"%s_%d_%d",lineArgs.geounit_filename_arg,t,outputcount);
         tofile(geounitfilename,myD,filetype);
        }
      }
    }
  }
}







bool hasEnding (std::string const &fullString, std::string const &ending) {
 if (fullString.length() >= ending.length()) {
   return (0 == fullString.compare (fullString.length() - ending.length(), ending.length(), ending));
  } else {
   return false;
  }
}


int main(int argc, char* argv[])
{
 gengetopt_args_info lineArgs;
 if (cmdline_parser(argc, argv, &lineArgs)) {
   cmdline_parser_print_help();
   return 1;
  }

 int seedoffset=lineArgs.seed_arg;



 //testsuite is not thread-safe:
#if defined(_OPENMP)
 if (lineArgs.testsuite_flag)
  omp_set_num_threads(1);
#endif



 //for Metropolis-Hastings but with uniform hard-constraint sampling in restriction
 if (lineArgs.compact_lambda_given || lineArgs.pop_lambda_given){
   if (!lineArgs.compact_lambda_given || !lineArgs.pop_lambda_given){
     cerr << "ERROR: either both or neither of --compact-lambda and --pop-lambda must be given."<<endl;
     exit(-1);
    }
  }

 bool intermediate_output=(! lineArgs.only_end_flag);



 g_debuglevel=lineArgs.debug_level_arg;

 geounit * pr; //array of geounits

 char myfilename[100];
 if (ends_with(lineArgs.filename_arg,".txt") || ends_with(lineArgs.filename_arg,".pct"))
  sprintf(myfilename,"%s",lineArgs.filename_arg);
 else
  sprintf(myfilename,"%s.pct",lineArgs.filename_arg);

 ifstream myfile (myfilename);
 string line;
 if (!myfile.good()){
   cerr << "ERROR with file"<<endl;
   exit(-1);
  }

 getline (myfile,line); //file header
 bool integerfile=hasEnding(line,"int");
 if (integerfile)
  cout <<"integerfile"<<endl;
 int filetype=1;
 if (!strstr(line.c_str(), "precinctlistv01")){ //nolines
   filetype=2;
   if (!strstr(line.c_str(), "precinctlistv02")){ //counties
     filetype=3;
     if (!strstr(line.c_str(), "precinctlistv03")){ //counties munis
       filetype=4;
       if (!strstr(line.c_str(), "precinctlistv04")){ //counties munis clusters
         filetype=5;
         if (!strstr(line.c_str(), "precinctlistv05")){ //cluster counties munis clusters
           filetype=6;
           if (!strstr(line.c_str(), "precinctlistv06")){ //cluster counties munis clusters incumbents
             string precclustercountymuni_header ("precinctlistv06");
             cerr << "ERROR: incorrect file header at pos " << precclustercountymuni_header.compare(line) << endl;
             exit(-1);
            }
          }
        }
      }
    }
  }


 if ((lineArgs.counties_flag || lineArgs.county_traversals_flag) && filetype<2){
   cerr << "ERROR: preserving counties or traversals requires a v02 or v03 or v04 file with county data!"<<endl;
   exit(-1);
  }
 if (lineArgs.muni_flag && filetype<3){
   cerr << "ERROR: preserving preserved municipalities requires a v03 or v04 file with county/muni data!"<<endl;
   exit(-1);
  }
 //  g_MAXMUNIS=lineArgs.max_munis_arg+1;

 if (lineArgs.incumbents_flag && filetype<6){
   cerr << "ERROR: preserving incumbents requires a v06 file with incumbent data!"<<endl;
   exit(-1);
  }





 getline(myfile,line); //#geounits
 g_NUMGEOS=atoi(line.c_str());
 if (intermediate_output)
  cout << "We have "<<g_NUMGEOS<<" geounits in total."<<endl;
 if (filetype>=4){
   getline(myfile,line); //#districts
   g_NUMDISTRICTS=atoi(line.c_str());
   getline(myfile,line); //#clusters
   g_NUMCLUSTERS=atoi(line.c_str());
  }
 else
  g_NUMCLUSTERS=1;
 if (filetype>=5){
   getline(myfile,line); //#precincts
   g_NUMPRECINCTS=atoi(line.c_str());
  }
 else
  g_NUMPRECINCTS=0;

 assert(g_NUMCLUSTERS>=1);

 cout << "we have "<<g_NUMCLUSTERS<< " cluster";
 if (g_NUMCLUSTERS>1)
  cout <<"s";
 cout <<" of geounits" <<endl;


 pr=new geounit[g_NUMGEOS+g_NUMCLUSTERS];  //one extra geounit object for each boundary


 //upper bound on crossing edges.

 getline(myfile,line);  //skip data header

 bool districtpresentinfile[MAXDISTRICTS]={false}; //track which districts we have

 int I=0;
 int sum=0;
 g_MAXDEGREE=0;
 while (getline(myfile,line) && I<g_NUMGEOS){
   vector<string> dataline;
   Tokenize(line.c_str(), dataline, "\t", true);
   int targetlength=9;
   if (filetype>=2) //counties
    targetlength+=1; //county column
   if (filetype>=3)
    targetlength+=2; //muni, muni area columns
   if (filetype>=4)
    targetlength+=1; //cluster columns
   if (filetype>=5)
    targetlength+=1; //precinct columns
   if (filetype>=6)
    targetlength+=1; //incumbent columns

   if (dataline.size()!=targetlength){
     cerr << "ERROR: shouldn't there be "<<targetlength<<" chunks per line?"<<endl;
     cerr << "there are "<<dataline.size()<<" at I="<<I<<endl;
     exit(-1);
    }
   if (atoi(dataline[0].c_str())!=I){
     cerr << "ERROR: line numbers don't match"<<endl;
     exit(-1);
    }
   pr[I]=geounit(dataline,lineArgs.flip_flag,filetype, integerfile);
   districtpresentinfile[pr[I].original_district]=true;
   sum+=pr[I].degree;
   I++;
  }
 assert(0<g_MAXDEGREE && g_MAXDEGREE<=MAXDEGREE);
 if (g_debuglevel>1)
  cout << "Max degree is "<<g_MAXDEGREE<<endl;

 if (I<g_NUMGEOS || !myfile.eof() ){
   cerr << "ERROR: Line numbers don't match geounit count"<<endl;
   exit(-1);
  }


 //create backneighbor lists:
 computeselves(pr);

 //the following deals with input files with nonconsecutive district numbers

 int districtcount=0;
 int last=0;
 for (int i=0; i<MAXDISTRICTS; i++){
   if (districtpresentinfile[i]){
     districtcount++;
     last=i;
    }
  }
 if (filetype>=4){
   if (g_NUMDISTRICTS!=districtcount){
     cerr << "ERROR: file says we have "<<g_NUMDISTRICTS<<" districts but file contains "<<districtcount<< " distinct districts"<<endl;
     exit(-1);
    }
  }
 else
  g_NUMDISTRICTS=districtcount;

 cout << "we have "<<g_NUMDISTRICTS<< " districts"<<endl;
 assert(g_NUMDISTRICTS<MAXDISTRICTS);

 if (lineArgs.seats_flag && g_NUMDISTRICTS<15){
   cerr <<"Should not use seats arg with small number of districts (exiting)"<<endl;
   exit(-1);
  }

 /////following code deals with nonconsecutive district numbers
 int * realdistrict; //track global district numbers
 int * districtindex; //return consecutive district number
 realdistrict=new int[districtcount];
 districtindex=new int[last+1];
 districtcount=0;
 for (int i=0; i<=last; i++){
   if (districtpresentinfile[i]){
     realdistrict[districtcount]=i;
     districtindex[i]=districtcount;
     districtcount++;
    }
  }
 for (int i=0; i<g_NUMGEOS; i++) //renumber consecutively
  pr[i].original_district=districtindex[ pr[i].original_district ];

 /////////////////////////





 bool output_freeze_lists=(g_debuglevel>0);

 if (lineArgs.counties_flag)
  freezegeounitsbycounty(pr,output_freeze_lists);

 s_array<int> supersetmuni(g_NUMDISTRICTS); // for each district, the index of muni completely containing it, or 0
 for (int i=0; i<g_NUMDISTRICTS; i++)
  supersetmuni[i]=0;

 if (lineArgs.muni_flag){
   freezegeounitsbymuni(pr, lineArgs.muni_eps_arg, realdistrict,output_freeze_lists, lineArgs.loose_munis_flag);
   findsupermunis(pr, supersetmuni, lineArgs.muni_eps_arg, realdistrict);
  }

 if (lineArgs.incumbents_flag)
  processincumbents(pr,output_freeze_lists);

 int numberfrozen=lineArgs.freeze_given;
 g_MAXEDGES=2*(2*(g_NUMGEOS+g_NUMDISTRICTS)+(g_NUMDISTRICTS-numberfrozen)-6);

 s_array<bool> frozen_districts=s_array<bool>(g_NUMDISTRICTS);  //which districts are frozen

 for (int k=0; k<g_NUMDISTRICTS; k++)
  frozen_districts[k]=0;                       //zero them

 for (int i=0; i<numberfrozen; i++){
   if (lineArgs.freeze_arg[i]<0 ||
       lineArgs.freeze_arg[i]>=MAXDISTRICTS ||
       !districtpresentinfile[lineArgs.freeze_arg[i]]){
     cerr << "ERROR: district "<<lineArgs.freeze_arg[i]<<" is not present in file"<<endl;
     exit(-1);
    }
   frozen_districts[districtindex[lineArgs.freeze_arg[i]]]=1;//set to one the given frozen
  }


 bool multilevel=(lineArgs.precincts_flag);
 if (multilevel && filetype<5){
   cerr << "ERROR: precinct split preservation requires v05 file"<<endl;
   exit(-1);
  }

 Ddata firstD(pr, frozen_districts, multilevel, lineArgs.cut_slack_arg);
 if (multilevel && lineArgs.splits_given)
  firstD.swapproposals->set_thresh(lineArgs.splits_arg);
 if (g_debuglevel>0){
   for (int k=0; k<g_NUMDISTRICTS; k++)
    cout << "district "<<realdistrict[k]<<" has " <<firstD.votesA[k]<<" voteA and "<<firstD.votesB[k]<<" voteB.  share is "<<firstD.Ashare.at(k)<<endl;
  }
 if (intermediate_output){
   int Avotes=0;
   int Bvotes=0;
   for (int k=0; k<g_NUMDISTRICTS; k++){
     Avotes+=firstD.votesA[k];
     Bvotes+=firstD.votesB[k];
    }
   if (g_debuglevel>0){
     cout << "A has "<<Avotes <<" votes"<<endl;
     cout << "B has "<<Bvotes <<" votes"<<endl;
     cout << "avg population is "<<firstD.avgpop<<endl;
    }
  }

 mt19937_64 gen(314159265358979-1+10000000*seedoffset);


 if (lineArgs.svg_filename_given){
   char svgfilename[100];
   sprintf(svgfilename,"%s_%d_%.3d.svg",lineArgs.svg_filename_arg,0,0);
   char inputsvgfilename[100];
   if (lineArgs.inputsvg_filename_given)
    sprintf(inputsvgfilename,"%s",lineArgs.inputsvg_filename_arg);
   else
    sprintf(inputsvgfilename,"%s.svg",lineArgs.filename_arg);
   tosvg(svgfilename,inputsvgfilename,firstD,lineArgs.svg_firstline_arg);
  }


 /////////////INITIAL CONNECTIVITY CHECK///////
 int count=0;
 if (g_debuglevel>1)
  cout << "connectivity check..."<<endl;
 for (int k=0; k<g_NUMDISTRICTS; k++){
   int localcount=0;
   queue <int> BFSqueue;
   for (int i=0; i<g_NUMGEOS; i++){
     if (pr[i].original_district==k){
       BFSqueue.push(i);
       pr[i].giant=true;
       count++; localcount++;
       break;
      }
    }
   while (!BFSqueue.empty()){
     int myindex=BFSqueue.front();
     BFSqueue.pop();
     assert(pr[myindex].giant==true);
     for (int j=0; j<pr[myindex].degree; j++){
       if (pr[pr[myindex].neighbors[j]].original_district==pr[myindex].original_district && pr[pr[myindex].neighbors[j]].giant==false){
         pr[pr[myindex].neighbors[j]].giant=true; count++; localcount++;
         BFSqueue.push(pr[myindex].neighbors[j]);
        }
      }
    }
   if (g_debuglevel>1)
    cerr << "district "<<k<< " giant has "<<localcount<<" geounits."<<endl;
  }

 bool passed=true;
 for (int v=0; v<g_NUMGEOS; v++){
   if (pr[v].giant==false){
     passed=false;
     cerr << "ERROR: Geounit "<<v<< " is not in its giant!"<<endl;
     cerr << "It's giant contains geounits:"<<endl;
     for (int u=0; u<g_NUMGEOS; u++){
       if (pr[u].giant==true && pr[u].original_district==pr[v].original_district)
        cerr << u << ", ";
      }
     cerr << endl;
    }
  }

 if (passed){
   assert(count==g_NUMGEOS);
   if (g_debuglevel>1)
    cout << "passed connectivity check" <<endl;
  }
 else
  exit(-1);
 ///////////END INITIAL CONNECTIVITY CHECK//////////////




 if ((int) (lineArgs.steps_given==true) + (int) (lineArgs.int_steps_given==true) != 1){
   cerr << "ERROR: exactly one of --steps and --int_steps must be given."<<endl;
   exit(-1);
  }


 int64_t steps;
 int64_t my_period_arg;
 int64_t period;
 int64_t periodS;
 if (lineArgs.steps_given){
   steps=pow(2,lineArgs.steps_arg-lineArgs.steplength_arg);
   if (lineArgs.period_given)
    my_period_arg=lineArgs.period_arg;
   else
    my_period_arg=lineArgs.steps_arg-2;
   period=max(1.5,pow(2,min(lineArgs.steps_arg,my_period_arg)-lineArgs.steplength_arg)+.5);
   periodS=max(1L,steps/period);
  }
 else{
   assert(lineArgs.int_steps_given);
   steps=lineArgs.int_steps_arg;
   period=steps;
   periodS=1L;
  }

 cout << "period is " << period <<" and periodS is " <<periodS<<endl;


 int T; //threads
 if (!strcmp(lineArgs.statistic_arg,"rootep"))   {
   StatTest=rootep;
   T=1;
  }
 else if (!strcmp(lineArgs.statistic_arg,"twoep"))   {
   StatTest=twoep;
   T=2;
  }
 else if (!strcmp(lineArgs.statistic_arg,"outlier"))   {
   StatTest=outlier;
   T=lineArgs.threads_arg;
  }
 else if (!strcmp(lineArgs.statistic_arg,"starsplit"))   {
   StatTest=starsplit;
   T=lineArgs.threads_arg;
  }
 else{
   cerr << "Unknown Statistical Test requested!!\n";
   exit(-1);
  }


 outputbag Out(lineArgs, firstD, gen, realdistrict);
 condbag Tests(lineArgs, firstD, realdistrict);



 s_array<datacount> threadcounts(T);

 //filling in first step...
 s_array<int64_t> step_pos(T);

 int64_t steplength=pow(2,lineArgs.steplength_arg);  //Markov Chain exponent
 s_double p=((s_double) firstD.swapproposals->count())/g_MAXEDGES;
 std::geometric_distribution<int64_t> revisits(p);
 if (StatTest!=outlier){
   int initialsteps=revisits(gen);
   int initialvisits=1+initialsteps/steplength;
   for (int t=0; t<T; t++){
     step_pos[t]=initialsteps%steplength;
    }


   if (g_debuglevel>0){
    cout << "first edgeset.count() is "<<firstD.swapproposals->count()<<endl;\
    cout << "1/p is "<<1/p<<endl;
    }
   if (intermediate_output){
     cout << "first p is "<<p<<endl;
     cout << "initialvisits is "<<initialvisits<<endl;
    }
   threadcounts[0]=datacount(initialvisits);
   for (int t=1; t<T; t++)
    threadcounts[t]=datacount();
  }

 else{ //StatTest==outlier
   if (g_debuglevel>0){
    cout << "first edgeset.count() is "<<firstD.swapproposals->count()<<endl;
    cout << "1/p is "<<1/p<<endl;
    }
   for (int t=0; t<T; t++){
     int initialsteps=revisits(gen);
     int initialvisits=1+initialsteps/steplength;
     step_pos[t]=initialsteps%steplength;
     if (g_debuglevel>0)
      cout << "thread "<<t<<" has "<<initialvisits<<" initial visits"<<endl;

     if (intermediate_output){
       cout << "Thread "<<t<<" initial visits is "<<initialvisits<<endl;
      }
     threadcounts[t]=datacount(initialvisits);
    }
  }



 //random sources:
 s_array<mt19937_64> threadgen(T);
 for (int t=0; t<T; t++)
  threadgen[t]=mt19937_64(31415926535897+t+seedoffset);


 cout <<endl<< "Beginning trajectory!!"<<endl<<endl;
 //MAIN LOOP
 if (StatTest==rootep){
   int t=0;
   Ddata myD=firstD;
   int outputcount=0;
   int64_t my_bigsteps=0;
   for (int i=1; i<=periodS; i++){
     MainLoop(myD, Out, threadcounts[0], Tests, supersetmuni, lineArgs.muni_eps_arg, lineArgs, filetype, steps, t, threadgen[t], outputcount, step_pos[0], my_bigsteps,period);
     outputcount++;
     cout << Out.forscreen(threadcounts[0], myD).str();
    }
  }


 else if (StatTest==twoep){
   s_array<int> outputcount(T);
   for (int t=0; t<T; t++)
    outputcount[t]=0;

#pragma omp parallel for
   for (int t=0; t<T; t++){
     Ddata myD=firstD;
     int64_t my_bigsteps=0;
     MainLoop(myD, Out, threadcounts[t], Tests, supersetmuni, lineArgs.muni_eps_arg, lineArgs, filetype, steps, t, threadgen[t], outputcount[t], step_pos[t], my_bigsteps,period);
     if (intermediate_output && t==0)
      cout << "Thread "<< t<< " reporting. " << endl << Out.forscreen(threadcounts[t], myD).str()<<endl;
    }
  }


 else if (StatTest==outlier){
   s_array<int64_t> my_bigsteps(T);
   s_array<int> outputcount(T);
   s_array<Ddata> threadD=s_array<Ddata>(T);
   for (int t=0; t<T; t++){
     my_bigsteps[t]=0;
     threadD[t]=firstD;
     outputcount[t]=0;
    }


   for (int i=1; i<=periodS; i++){
#pragma omp parallel for
     for (int t=0; t<T; t++){
       MainLoop(threadD[t], Out, threadcounts[t], Tests, supersetmuni, lineArgs.muni_eps_arg, lineArgs, filetype, i*period, t, threadgen[t], outputcount[t], step_pos[t], my_bigsteps[t],0);
       outputcount[t]++;
       cout << "Thread "<<t<<" reporting."<<endl<<Out.forscreen(threadcounts[t],threadD[t]).str()<<endl;
       if (t==0)
        tosvg(threadD[t],lineArgs,t,outputcount[t]);
      }
     Out.outlier_output(threadcounts, T, lineArgs.doubling_rhos_flag, lineArgs.num_rhos_arg, lineArgs.alpha_mult_arg);
     cout << "end of period "<< i <<endl<<endl;
    }
  }


 else if (StatTest==starsplit){
   s_array<int> outputcount(T);
   for (int t=0; t<T; t++)
    outputcount[t]=0;


   uniform_int_distribution<int64_t> Lintdist(0,steps-1);
   s_array<int64_t> threadsteps=s_array<int64_t>(T);
   for (int t=0; t<T; t++)
    threadsteps[t]=steps;
   threadsteps[0]=Lintdist(gen);
   threadsteps[1]=steps-threadsteps[0];
   if (intermediate_output){
     cout << "We'll branch after "<<threadsteps[0]<<" steps"<<endl;
    }
   s_array<Ddata> threadD=s_array<Ddata>(T);

   threadD[0]=firstD;    //First threads begin from initial state;
   threadD[1]=firstD;
#pragma omp parallel for
   for (int t=0; t<2; t++){
     int64_t my_bigsteps=0;
     MainLoop(threadD[t], Out, threadcounts[t], Tests, supersetmuni, lineArgs.muni_eps_arg, lineArgs, filetype, threadsteps[t], t, threadgen[t], outputcount[t], step_pos[t], my_bigsteps,0);
    }
   for (int t=2; t<T; t++){
     threadD[t]=threadD[0]; //remaining threads branch from end of one of the previous threads
    }
#pragma omp parallel for
   for (int t=2; t<T; t++){
     int64_t my_bigsteps=0;
     MainLoop(threadD[t], Out, threadcounts[t], Tests, supersetmuni, lineArgs.muni_eps_arg, lineArgs, filetype, threadsteps[t], t, threadgen[t], outputcount[t], step_pos[t], my_bigsteps,0);
    }
   Out.output(threadcounts, threadD[0], T);
   //cout << "Thread "<< T-1<< " reporting. " << endl << Out.forscreen(threadcounts[T-1], threadD[T-1]).str()<<endl;
  }



 if (lineArgs.testsuite_flag){
   hash<string> hasher;
   int distinct=0;
   stringstream output;
   unordered_map<string,int>::iterator it=g_THECOUNT.begin();\
   while (it!=g_THECOUNT.end()){
     distinct++;
     output << it->second << " ";
     output << hasher(it->first)<<endl;
     it++;
    }
   output << endl <<"there are "<<distinct<< " unique districtings seen"<<endl;
   cout << output.str();
  }
 delete [] pr;
}
